<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
        xmlns:xalan="http://xml.apache.org/xslt"
        xmlns:stu="org.ejen.ext.StringUtil"
        version="1.0">
<xsl:output method="text" encoding="iso-8859-1"/>
<xsl:template name="fileHeader">
/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * <xsl:value-of select="PACKAGE"/>/<xsl:value-of select="CLASS_NAME"/>Query.java
 *-----------------------------------------------------------------------------
 */

package <xsl:value-of select="translate(PACKAGE,'/','.')"/>;

import org.enhydra.dods.DODS;
import com.lutris.dods.builder.generator.query.*;
import com.lutris.dods.builder.generator.dataobject.GenericDO;
import com.lutris.appserver.server.sql.*;
import org.enhydra.dods.cache.LRUCache;
import org.enhydra.dods.cache.QueryCacheItem;
import org.enhydra.dods.cache.QueryCacheItemImpl;
import org.enhydra.dods.cache.QueryResult;
import org.enhydra.dods.cache.DOShell;
import org.enhydra.dods.cache.Condition;
import org.enhydra.dods.statistics.Statistics;
import org.enhydra.dods.statistics.TableStatistics;
import org.enhydra.dods.statistics.QueryCacheStatistics;

import java.sql.*;
import java.util.*;
import java.math.*;
import java.util.Date;  // when I say Date, I don't mean java.sql.Date

// WebDocWf extension for DODS row instance based security
// The following line has been added
import org.webdocwf.dods.access.*;
// end of WebDocWf extension for DODS row instance based security

/**
 * <xsl:value-of select="CLASS_NAME"/>Query is used to query the <xsl:value-of select="TABLE_NAME"/> table in the database.&lt;BR&gt;
 * It returns objects of type <xsl:value-of select="CLASS_NAME"/>DO.
 * &lt;P&gt;
 * General usage:
 * &lt;P&gt;
 *     In DODS:
 *        Create a Data Object named "Dog",
 *        and create an Attribute named "Name",
 *        and set that Attribute to "Can be queried."
 *        DODS will then generate the method DogQuery.setQueryName().
 * &lt;P&gt;
 *     In your Business Layer, prepare the query:&lt;BR&gt;
 * &lt;P&gt;&lt;PRE&gt;
 *             DogQuery dq = new DogQuery();
 *             dq.setQueryName("Rex")
 *             if ( Rex is a reserved name )
 *                 dq.requireUniqueInstance();
 * &lt;/PRE&gt;
 *     Then, get the query results one of two ways:
 * &lt;P&gt;
 *         #1:&lt;PRE&gt;
 *             String names = "";
 *             DogDO[] dogs = dq.getDOArray();
 *             for ( int i = 0; i &lt; dogs.length; i++ ) {
 *                 names += dogs[i].getName() + " ";
 *             }
 * &lt;/PRE&gt;
 *      or #2:&lt;PRE&gt;
 *             String names = "";
 *             DogDO dog;
 *             while ( null != ( dog = dq.getNextDO() ) ) {
 *                 names += dog.getName() + " ";
 *             }
 * &lt;/PRE&gt;
 * &lt;P&gt;
 *     Note:   If &lt;CODE&gt;requireUniqueInstance()&lt;/CODE&gt; was called,
 *             then &lt;CODE&gt;getDOArray()&lt;/CODE&gt; or &lt;CODE&gt;getNextDO()&lt;/CODE&gt;
 *             will throw an exception if more than one "Rex" was found.
 * &lt;P&gt;
 *     Note:   Results of the query will come from the Data Object cache if:
 *             -  The cache is available.
 *             -  Matches were found in the cache.
 *             -  No other tables (Data Objects of other types) were involved
 *                in the query.
 *                This can happen if you extend the &lt;CODE&gt;DogQuery&lt;/CODE&gt; class
 *                and you make calls to the &lt;CODE&gt;QueryBuilder&lt;/CODE&gt; object
 *                to add SQL involving other tables.
 *             If any of these conditions is not true,
 *             then any results from the query will come from the database.
 * &lt;P&gt;
 *     To reuse the query object, call:
 * &lt;P&gt;&lt;PRE&gt;
 *             dq.reset();
 * &lt;/PRE&gt;
 * @author <xsl:value-of select="AUTHOR"/>
 * @version $Revision: 1.7 $
 */
public class <xsl:value-of select="CLASS_NAME"/>Query implements Query {

    private QueryBuilder builder;

    /**
     * logical name of the database for which <xsl:value-of select="CLASS_NAME"/>Query object has been created
     */
    private String logicalDatabase;

	private ResultSet resultSet = null;
	private boolean uniqueInstance  = false;
	private <xsl:value-of select="CLASS_NAME"/>DO[] DOs = null;
	private int arrayIndex = -1;
	private boolean needToRun = true;
	private LinkedHashMap cacheHits = null;
	private boolean isQueryByOId = false;
	private boolean hitDb = false;
    private boolean userHitDb = false;

	private QueryCacheItem queryItem = null;
	private String currentHandle = null;

    /**
     * Public constructor.
     */
    public <xsl:value-of select="CLASS_NAME"/>Query() {
        builder = new QueryBuilder( &quot;<xsl:value-of select="TABLE_NAME"/>&quot;, &quot;<xsl:value-of select="TABLE_NAME"/>.*&quot; );
        String dbName = <xsl:value-of select="CLASS_NAME"/>DO.getLogicalDBName();
        try {
            String vendor = DODS.getDatabaseManager().logicalDatabaseType(dbName);
            if (vendor != null) {
                builder.setDatabaseVendor(vendor);
                logicalDatabase = dbName;
            } else {
                builder.setDatabaseVendor();
                logicalDatabase =  DODS.getDatabaseManager().getDefaultDB();
            }
        } catch (Exception e) {
            builder.setDatabaseVendor();
            logicalDatabase =  DODS.getDatabaseManager().getDefaultDB();
        }
        builder.setUserStringAppendWildcard( false );
        builder.setUserStringTrim( false );
        reset();
    }

<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * Public constructor.
     *
     * @param dbName
     *   Logical name of the database from which <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     *
     * @exception AccessException
     *   The user is not allowed to create a query
     */
    public <xsl:value-of select="CLASS_NAME"/>Query(String dbName) throws DatabaseManagerException {
        builder = new QueryBuilder( &quot;<xsl:value-of select="TABLE_NAME"/>&quot;, &quot;<xsl:value-of select="TABLE_NAME"/>.*&quot; );
        try {
            String vendor = DODS.getDatabaseManager().logicalDatabaseType(dbName);
            if (vendor != null) {
                builder.setDatabaseVendor(vendor);
                logicalDatabase = dbName;
            } else {
                builder.setDatabaseVendor();
                logicalDatabase =  DODS.getDatabaseManager().getDefaultDB();
            }
        } catch (SQLException e) {
            throw new DatabaseManagerException ("INTERNAL ERROR: unexpected SQLException" );
        }
        builder.setUserStringAppendWildcard( false );
        builder.setUserStringTrim( false );
        reset();
    }
</xsl:if>

    /**
     * Return logical name of the database that <xsl:value-of select="CLASS_NAME"/>Query object uses
     *
     * @return param logical database name
     *
     */
    public String getLogicalDatabase() {
        return logicalDatabase;
    }

    /**
     * Change logical database to another logical database (which name is dbName)
     *
     * @param dbName the logical name of the database
     *
     */
    public void setLogicalDatabase(String dbName) throws SQLException, DatabaseManagerException {
        // Support for DB independence added by Tanja Jovanovic 2002.12.03.
        String vendor = DODS.getDatabaseManager().logicalDatabaseType(dbName);
        if (vendor != null) {
            builder.setDatabaseVendor(vendor);
            logicalDatabase = dbName;
        } else {
            builder.setDatabaseVendor();
            logicalDatabase =  DODS.getDatabaseManager().getDefaultDB();
        }
        reset();
        // end of support for DB independence
    }

    //  WebDocWf extension for unique query results without SQL DISTINCT
    //  The following lines has been added:
    private boolean unique = false;

    /**
     * Set the unique flag of the query
     *
     * @param newUnique The unique flag for the query
     *
     * WebDocWf extension
     *
     */
    public void setUnique(boolean newUnique) { unique = newUnique; }

    /**
     * Get the unique flag of the query
     *
     * @return The unique flag of the query
     *
     * WebDocWf extension
     *
     */
    public boolean getUnique() { return unique; }
    //  end of WebDocWf extension for unique query results without SQL DISTINCT

    //  WebDocWf extension for skipping the n first rows of the result
    //  The following lines has been added:
    private int readSkip = 0;

    /**
     * Set the readSkip number of the query
     *
     * @param newreadSkip The number of results to skip
     *
     * WebDocWf extension
     *
     */
    public void setReadSkip(int newReadSkip) { readSkip = newReadSkip; }

    /**
     * Get the readSkip number of the query
     *
     * @return The number of rows which are skipped
     *
     * WebDocWf extension
     *
     */
    public int getReadSkip() { return readSkip; }
    // end of WebDocWf extension for skipping the n first rows of the result

    // WebDocWf extension for select rowcount limit
    // The following lines has been added:
    private int databaseLimit = 0;

    /**
     * Set the database limit of the query
     *
     * @param newLimit The limit for the query
     *
     * WebDocWf extension
     *
     */
    public void setDatabaseLimit(int newLimit) { databaseLimit = newLimit; }

    /**
     * Get the database limit of the query
     *
     * @return The database limit of the query
     *
     * WebDocWf extension
     *
     */
    public int getDatabaseLimit() { return databaseLimit; }

    private boolean databaseLimitExceeded = false;

    /**
     * Get the database limit exceeded flag of the query.
     *
     * @return The database limit exceeded flag of the query
     *   True if there would have been more rows than the limit, otherwise false.
     *
     * WebDocWf extension
     */
    public boolean getDatabaseLimitExceeded() { return databaseLimitExceeded; }
    // end of WebDocWf extension for select rowcount limit

    /**
     * Set that all queries go to database, not to cache.
     */
    public void hitDatabase() { userHitDb = true; }

    // WebDocWf extension for extended wildcard support
    // The following rows have been added:

    /**
     * Set user string wildcard.
     *
     * @param newUserStringWildcard New user string wildcard.
     *
     * WebDocWf extension
     */
    public void setUserStringWildcard(String newUserStringWildcard) {
        builder.setUserStringWildcard( newUserStringWildcard );
    }

    /**
     * Set user string single wildcard.
     *
     * @param newUserStringSingleWildcard New user string single wildcard.
     *
     * WebDocWf extension
     */
    public void setUserStringSingleWildcard(String newUserStringSingleWildcard) {
        builder.setUserStringSingleWildcard( newUserStringSingleWildcard );
    }

    /**
     * Set user string single wildcard escape.
     *
     * @param newUserStringSingleWildcardEscape New user string single wildcard escape.
     *
     * WebDocWf extension
     */
    public void setUserStringSingleWildcardEscape(String newUserStringSingleWildcardEscape) {
        builder.setUserStringSingleWildcardEscape( newUserStringSingleWildcardEscape );
    }

    /**
     * Set user string wildcard escape.
     *
     * @param newUserStringWildcardEscape New user string wildcard escape.
     *
     * WebDocWf extension
     */
    public void setUserStringWildcardEscape(String newUserStringWildcardEscape) {
        builder.setUserStringWildcardEscape( newUserStringWildcardEscape );
    }

    /**
     * Set user string append wildcard.
     *
     * @param userStringAppendWildcard New user string append wildcard.
     *
     * WebDocWf extension
     */
    public void setUserStringAppendWildcard(boolean userStringAppendWildcard ) {
        builder.setUserStringAppendWildcard( userStringAppendWildcard );
    }

     /**
     * Set user string trim.
     *
     * @param userStringTrim New user string trim.
     *
     * WebDocWf extension
     */
    public void setUserStringTrim(boolean userStringTrim ) {
        builder.setUserStringTrim( userStringTrim );
    }

    /**
     * Get user string wildcard.
     *
     * @return User string wildcard.
     *
     * WebDocWf extension
     */
    public String getUserStringWildcard() {
        return builder.getUserStringWildcard();
    }

    /**
     * Get user string single wildcard.
     *
     * @return User string single wildcard.
     *
     * WebDocWf extension
     */
    public String getUserStringSingleWildcard() {
        return builder.getUserStringSingleWildcard();
    }

    /**
     * Get user string single wildcard escape.
     *
     * @return User string single wildcard escape.
     *
     * WebDocWf extension
     */
    public String getUserStringSingleWildcardEscape() {
        return builder.getUserStringSingleWildcardEscape();
    }

    /**
     * Get user string wildcard escape.
     *
     * @return User string wildcard escape.
     *
     * WebDocWf extension
     */
    public String getUserStringWildcardEscape() {
        return builder.getUserStringWildcardEscape();
    }

    /**
     * Get user string append wildcard.
     *
     * @return User string append wildcard.
     *
     * WebDocWf extension
     */
    public boolean getUserStringAppendWildcard() {
        return builder.getUserStringAppendWildcard();
    }

    /**
     * Get user string trim.
     *
     * @return User string trim.
     *
     * WebDocWf extension
     */
    public boolean getUserStringTrim() {
        return builder.getUserStringTrim();
    }
    // end of WebDocWf extension for extended wildcard support

	/**
	 * Perform the query on the database, and prepare the
	 * array of returned DO objects.
	 *
	 * @exception DataObjectException If a database access error occurs.
	 * @exception NonUniqueQueryException If too many rows were found.
	 */
	private void getQueryByOIds(Vector DOs)
	throws DataObjectException {
		if (DOs.size() == 0)
			return;
		<xsl:value-of select="CLASS_NAME"/>DO DO = null;
		DOShell shell = null;
    	Date startQueryTime = new Date();
    	long queryTime = 0;
		for (int i=0; i&lt;DOs.size(); i++) {
			shell = (DOShell)DOs.elementAt(i);
			DO = <xsl:value-of select="CLASS_NAME"/>DO.createExisting(<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">logicalDatabase,</xsl:if>shell.handle);
			shell.dataObject = DO;
        }
		Date stopQueryTime = new Date();
        queryTime = stopQueryTime.getTime() - startQueryTime.getTime();
	    <xsl:value-of select="CLASS_NAME"/>DO.statistics.updateQueryByOIdAverageTime((new Long(queryTime)).intValue(),DOs.size());
	}



	/**
	 * Perform the query on the database, and prepare the
	 * array of returned DO objects.
	 *
	 * @exception DataObjectException If a database access error occurs.
	 * @exception NonUniqueQueryException If too many rows were found.
	 */
	private void runQuery()
	throws DataObjectException, NonUniqueQueryException {
		needToRun = false;
		arrayIndex = -1;
		DBQuery dbQuery = null;
		Date startQueryTime = new Date();
		long queryTime = 0;
		boolean readDOs = false;
		try {
			QueryResult results = null;
			DOShell shell = null;
            if (<xsl:value-of select="CLASS_NAME"/>DO.cache != null) {
    			if (isQueryByOId) { // query by OId
    				results = new QueryResult();
    				if (currentHandle != null) { // select has one query by OId
   					    <xsl:value-of select="CLASS_NAME"/>DO DO = (<xsl:value-of select="CLASS_NAME"/>DO)<xsl:value-of select="CLASS_NAME"/>DO.cache.getDOByHandle(currentHandle);
    					if (DO != null) { // DO is found in the cache
    						if (queryItem.checkConditions(DO)){
    						    shell = new DOShell(DO);
    							results.DOs.add(shell);
    						}
                            ((QueryCacheStatistics)<xsl:value-of select="CLASS_NAME"/>DO.statistics).incrementCacheByOIdHitsNum();
    					} else { // DO isn't found in the cache
    						if (!<xsl:value-of select="CLASS_NAME"/>DO.cache.isFull())
    							hitDb = true;
    					}
    				} else { // select has more than one query by OId, there is no result for this query
    					hitDb = false;
    				}
    			} else { // other queries
    				String queryID = builder.getSQLwithParms()+"rs="+readSkip +"dl="+databaseLimit; // unique representation of query
    				if (hitDb) {
    					results = <xsl:value-of select="CLASS_NAME"/>DO.cache.getComplexQueryResults (logicalDatabase, queryID);
	                    ((TableStatistics)<xsl:value-of select="CLASS_NAME"/>DO.statistics).incrementComplexQueryNum();
    			    } else {
    					results = <xsl:value-of select="CLASS_NAME"/>DO.cache.getSimpleQueryResults (logicalDatabase, queryID);
	                    ((TableStatistics)<xsl:value-of select="CLASS_NAME"/>DO.statistics).incrementSimpleQueryNum();
                    }
    				if (results == null) { // query doesn't exist
    					queryItem.setQueryId(queryID);
    					if (hitDb)
    						<xsl:value-of select="CLASS_NAME"/>DO.cache.addComplexQuery(queryItem); // register complex query
    					else
    						<xsl:value-of select="CLASS_NAME"/>DO.cache.addSimpleQuery(queryItem); // register simple query
                        if (<xsl:value-of select="CLASS_NAME"/>DO.cache.isFull()) {
            				results = new QueryResult();
                        } else
    					    hitDb = true;
    				} else { // query found
        				if (hitDb)
    	                    ((QueryCacheStatistics)<xsl:value-of select="CLASS_NAME"/>DO.statistics).incrementComplexCacheHitsNum();
        			    else
    	                    ((QueryCacheStatistics)<xsl:value-of select="CLASS_NAME"/>DO.statistics).incrementSimpleCacheHitsNum();
                        int lazyTime = ((QueryCacheStatistics)<xsl:value-of select="CLASS_NAME"/>DO.statistics).getQueryByOIdAverageTime()*results.lazy.size();
                        if (lazyTime > queryItem.getTime())
        					hitDb = true;
        				else
    					    hitDb = false;
    				}
    			}
            } else
                hitDb = true;
			if ( hitDb || userHitDb) { // go to database
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
				if (logicalDatabase != null)
					dbQuery = <xsl:value-of select="CLASS_NAME"/>DO.createQuery(logicalDatabase);
				else</xsl:if>
					dbQuery = <xsl:value-of select="CLASS_NAME"/>DO.createQuery();
				dbQuery.query( this );    // invokes executeQuery
				results = new QueryResult();
				// WebDocWf extension
				// The following lines have been added:
				int tempReadSkip=0;
				boolean bHasMoreResults = false;
				if (! unique) {
					while ( (bHasMoreResults = resultSet.next()) &amp;&amp; (databaseLimit==0 || (results.DOs.size()&lt;databaseLimit) ) ) {
						<xsl:value-of select="CLASS_NAME"/>DO newDO;
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
						if (logicalDatabase != null)
							newDO = <xsl:value-of select="CLASS_NAME"/>DO.createExisting ( logicalDatabase, resultSet );
						else
</xsl:if>
							newDO = <xsl:value-of select="CLASS_NAME"/>DO.createExisting ( resultSet );
      								if (tempReadSkip &gt;= readSkip) {
      									newDO = <xsl:value-of select="CLASS_NAME"/>DO.addToCache(newDO);
      									queryItem.add(newDO);
      									shell = new DOShell(newDO);
      									results.DOs.add(shell);
      								} else {
      									tempReadSkip = tempReadSkip + 1;
      								}
					} // while
				}<xsl:if test="DO_IS_OID_BASED='true'"> else { // if (! unique)
					HashSet hsResult = new HashSet(databaseLimit * 2);
					while ( (bHasMoreResults = resultSet.next()) &amp;&amp; (databaseLimit==0 || (hsResult.size()&lt;databaseLimit) ) ) {
						<xsl:value-of select="CLASS_NAME"/>DO newDO;
						    newDO = <xsl:value-of select="CLASS_NAME"/>DO.createExisting ( resultSet );
								if (tempReadSkip &gt;= readSkip) {
									if (!hsResult.contains(newDO.getHandle())) {
										newDO = <xsl:value-of select="CLASS_NAME"/>DO.addToCache(newDO);
										queryItem.add(newDO);
										shell = new DOShell(newDO);
      									results.DOs.add(shell);
										hsResult.add(newDO.getHandle());
									} else {
										tempReadSkip = tempReadSkip +1;
									}
								} else {
									if (!hsResult.contains(newDO.getHandle()))
										tempReadSkip = tempReadSkip +1;
								}
                    }
				} // else (! unique)</xsl:if>
				if ((results.DOs.size()==databaseLimit)&amp;&amp; bHasMoreResults) {
					resultSet.close();
					databaseLimitExceeded = true;
				}
        		Date stopQueryTime = new Date();
                queryTime = stopQueryTime.getTime() - startQueryTime.getTime();
                queryItem.setTime((new Long(queryTime)).intValue());
			} else { // if ( hitDb )
				if (<xsl:value-of select="CLASS_NAME"/>DO.cache != null) {
//    				System.out.println(&quot;Pre <xsl:value-of select="CLASS_NAME"/>&quot;+results.toString(true));
    				if (<xsl:value-of select="CLASS_NAME"/>DO.cache.isFull()) {
    					if (readSkip&lt;cacheHits.size()) {
    						Vector vect = new Vector(cacheHits.values());
    						results.DOs = new Vector(vect.subList(readSkip, vect.size()));
                            readDOs = true;
    						// executeQuery saw cache hits
    					} else {
    						results.DOs = new Vector();
    					}
    				} else {
    					getQueryByOIds(results.lazy);
    				}
//    				System.out.println(&quot;Posle <xsl:value-of select="CLASS_NAME"/>&quot;+results.toString(true));
     				((QueryCacheStatistics)<xsl:value-of select="CLASS_NAME"/>DO.statistics).incrementCacheHitsNum();
//     				((QueryCacheStatistics)<xsl:value-of select="CLASS_NAME"/>DO.getStatistics()).show();
                }
			}
			// end of WebDocWf extension
			if ( results.DOs.size() &gt; 1 &amp;&amp; uniqueInstance )
				throw new NonUniqueQueryException("Too many rows returned from database" );
			DOs = new <xsl:value-of select="CLASS_NAME"/>DO [ results.DOs.size() ];
            if (readDOs)
    			for ( int i = 0; i &lt; results.DOs.size(); i++ )
    				DOs[i] = (<xsl:value-of select="CLASS_NAME"/>DO)results.DOs.elementAt(i);
            else
    			for ( int i = 0; i &lt; results.DOs.size(); i++ )
    				DOs[i] = (<xsl:value-of select="CLASS_NAME"/>DO)((DOShell)results.DOs.elementAt(i)).dataObject;
		    <xsl:value-of select="CLASS_NAME"/>DO.statistics.incrementQueryNum();
			arrayIndex = 0;
            if (!isQueryByOId)
			    <xsl:value-of select="CLASS_NAME"/>DO.statistics.updateQueryAverageTime((new Long(queryTime)).intValue());
			else
                <xsl:value-of select="CLASS_NAME"/>DO.statistics.incrementQueryByOIdNum();
		} catch ( SQLException se ) {
			if (null == se.getSQLState() ) {
				throw new DataObjectException("Unknown SQLException", se );
			}
			if ( se.getSQLState().startsWith("02") &amp;&amp; se.getErrorCode() == 100 ) {
				throw new DataObjectException("Update or delete DO is out of synch", se );
			} else if (     se.getSQLState().equals("S1000") &amp;&amp;     se.getErrorCode() == -268) {
				throw new DataObjectException("Integrity constraint violation", se );
			} else {
				throw new DataObjectException( "Data Object Error", se );
			}
		} catch ( ObjectIdException oe ) {
			throw new DataObjectException( "Object ID Error", oe );
		} catch ( DatabaseManagerException de ) {
			throw new DataObjectException( "Database connection Error", de );
		} finally {
			if ( null != dbQuery )
				dbQuery.release();
		}
	}

    /**
     * Limit the number of rows (DOs) returned.
     * NOTE: When setting a limit on rows returned by a query,
     * you usually want to use a call to an addOrderBy method
     * to cause the most interesting rows to be returned first.
<xsl:if test="not(CACHING='none' or DO_IS_OID_BASED='false')">
     * However, the DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.
     * So, setMaxRows also forces the query to hit the database.
</xsl:if>
     *
     * @param maxRows Max number of rows (DOs) returned.
     *
     * @exception DataObjectException If a database access error occurs.
     * @exception NonUniqueQueryException If too many rows were found.
     */
    public void setMaxRows( int maxRows )
    throws DataObjectException, NonUniqueQueryException {
        userHitDb = true;
        builder.setMaxRows( maxRows );
    }

    /**
     * Return array of DOs constructed from ResultSet returned by query.
     *
     * @return Array of DOs constructed from ResultSet returned by query.
     *
     * @exception DataObjectException If a database access error occurs.
     * @exception NonUniqueQueryException If too many rows were found.
     */
    public <xsl:value-of select="CLASS_NAME"/>DO[] getDOArray()
    throws DataObjectException, NonUniqueQueryException {
        if ( needToRun )
            runQuery();
        return DOs;
    }

    /**
     * Return successive DOs from array built from ResultSet returned by query.
     *
     * @return DOs from array built from ResultSet returned by query.
     *
     * @exception DataObjectException If a database access error occurs.
     * @exception NonUniqueQueryException If too many rows were found.
     */
    public <xsl:value-of select="CLASS_NAME"/>DO getNextDO()
    throws DataObjectException, NonUniqueQueryException {
        if ( needToRun )
            runQuery();
        if ( null == DOs ) {
            /* This should never happen.
             * runQuery() should either throw an exception
             * or create an array of DOs, possibly of zero length.
             */
            return null;
        }
        if ( arrayIndex &lt; DOs.length )
            return DOs[ arrayIndex++ ];
        return null;
    }

<xsl:if test="DO_IS_OID_BASED='true'">
    /**
     * Set the OID to query.
     * WARNING!  This method assumes that table &lt;CODE&gt;<xsl:value-of select="TABLE_NAME"/>&lt;/CODE&gt;
     * has a column named &lt;CODE&gt;"oid"&lt;/CODE&gt;.
     * This method is called from the DO classes to retrieve an object by id.
     *
     * @param oid The object id to query.
     */
    public void setQueryOId(ObjectId oid) {
        // Remove from cacheHits any DO that does not meet this
        // setQuery requirement.
		String handle = oid.toString();
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'"><xsl:text>
		</xsl:text>handle = getLogicalDatabase()+"."+handle;
</xsl:if>
        if (<xsl:value-of select="CLASS_NAME"/>DO.cache != null &amp;&amp; <xsl:value-of select="CLASS_NAME"/>DO.cache.isFull()) {
			isQueryByOId = true;
            <xsl:value-of select="CLASS_NAME"/>DO DO = (<xsl:value-of select="CLASS_NAME"/>DO)cacheHits.get(handle);
            if (DO != null) {
                cacheHits = new LinkedHashMap();
                cacheHits.put(handle,DO);
            }
        } else {
			if (isQueryByOId) { // query by OId already has been invoked
				if (!currentHandle.equals(handle)) // two different OIds are invoked
					currentHandle = null;
			} else {
				currentHandle =  handle;
				isQueryByOId = true;
			}
        }
        // Also prepare the SQL needed to query the database
        // in case there is no cache, or the query involves other tables.
        builder.addWhere( <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey, oid.toBigDecimal(), QueryBuilder.EQUAL );
        <xsl:value-of select="CLASS_NAME"/>DO.statistics.incrementQueryByOIdNum();
    }

    /**
     * Set the object handle to query.
     * This is a variant of setQueryOId().
     *
     * @param handle The string version of the id to query.
     */
    public void setQueryHandle(String handle)
    throws ObjectIdException {
        ObjectId oid = new ObjectId( handle );
        setQueryOId( oid );
    }
</xsl:if>

    /**
     * Set "unique instance" assertion bit.
     * The first call to the next() method will throw an exception
     * if more than one object was found.
     */
    public void requireUniqueInstance() {
        uniqueInstance = true;
    }

    /**
     * Reset the query parameters.
     */
    public void reset() {
        if (<xsl:value-of select="CLASS_NAME"/>DO.cache != null &amp;&amp; <xsl:value-of select="CLASS_NAME"/>DO.cache.isFull()) {
            cacheHits = new LinkedHashMap(<xsl:value-of select="CLASS_NAME"/>DO.cache.getCacheContent());
        }
        DOs = null;
        uniqueInstance  = false;
        needToRun       = true;
        builder.reset();
        if (<xsl:value-of select="CLASS_NAME"/>DO.cache != null)
            queryItem = <xsl:value-of select="CLASS_NAME"/>DO.cache.newQueryCacheItemInstance(logicalDatabase);
        else
            queryItem = new QueryCacheItemImpl(logicalDatabase);
        userHitDb = false;
    }

    /**
     * Return the appropriate QueryBuilder flag for selecting
     * exact matches (SQL '=') or inexact matches (SQL 'matches').
     *
     * @return boolean True if it is exact match, otherwise false.
     *
     */
    private boolean exactFlag( boolean exact ) {
        return exact ? QueryBuilder.EXACT_MATCH : QueryBuilder.NOT_EXACT;
    }

    //
    // Implementation of Query interface
    //

    /**
     * Method to query objects from the database.
     * The following call in runQuery()
     *          dbQuery.query( this );
     * causes the dbQuery object to invoke
     *      executeQuery()
     *
     * @param conn Handle to database connection.
     *
     * @return ResultSet with the results of the query.
     *
     * @exception java.sql.SQLException If a database access error occurs.
     */
    public ResultSet executeQuery(DBConnection conn)
    throws SQLException     {
        resultSet = builder.executeQuery( conn );
        return resultSet;
    }

    /**
     * WARNING!  This method is disabled.
     * It's implementation is forced by the Query interface.
     * This method is disabled for 2 reasons:
     * 1)  the getDOArray() and getNextDO() methods are better
     *     because they return DOs instead of JDBC objects.
     * 2)  the createExisting() method throws an exception
     *     that we cannot reasonably handle here,
     *     and that we cannot throw from here.
     *
     * @param rs JDBC result set from which the next object
     *   will be instantiated.
     *
     * @exception java.sql.SQLException
     *   If a database access error occurs.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an invalid object id was queried from the database.
     */
    public Object next(ResultSet rs)
    throws SQLException, ObjectIdException {
        // TODO: It would be nice to throw an unchecked exception here
        // (an exception that extends RuntimeException)
        // that would be guaranteed to appear during application testing.
        throw new ObjectIdException("next() should not be used.  Use getNextDO() instead." );
        //return null;
    }


    // WebDocWf extension for extended wildcard support
    // The following lines have been added:
    /**
     * Convert a String with user wildcards into a string with DB wildcards
     *
     * @param userSearchValue The string with user wildcards
     *
     * @return The string with DB wildcards
     *
     * WebDocWf extension
     *
     */
    public String convertUserSearchValue( String userSearchValue ) {
        return builder.convertUserSearchValue( userSearchValue );
    }

    /**
     * Check whether a string contains DB wildcards
     *
     * @param dbSearchValue The string with possible DB wildcards
     *
     * @return Whether a string contains DB wildcards
     *
     * WebDocWf extension
     *
     */
    public boolean containsWildcards( String dbSearchValue ) {
        return builder.containsWildcards( dbSearchValue );
    }
    // end of WebDocWf extension for extended wildcard support

    // WebDocWf extension for query row counter
    // The following lines have been added:
    /**
     * Get the rowcount of the query
     * If possible, do it without reading all rows
     *
     * @return The row count
     *
     * WebDocWf extension
     *
     */
    public int getCount()
    throws NonUniqueQueryException, DataObjectException, SQLException, DatabaseManagerException     {
        int rowCount=0;
        if (needToRun &amp;&amp; databaseLimit==0) {
            rowCount = selectCount();
        } else {
            if (needToRun) runQuery();
                rowCount = DOs.length;
        }
        return rowCount;
    }

    /**
     * Get the rowcount of the query by using count(*) in the DB
     *
     * @return The row count
     *
     * WebDocWf extension
     *
     */
    public int selectCount()
    throws SQLException, DatabaseManagerException {
        int rowCount=0;
        String tempClause = builder.getSelectClause();
        builder.setSelectClause(" count(*) as \"counter\" ");
        DBQuery dbQuery;
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
        if (logicalDatabase != null)
            dbQuery = <xsl:value-of select="CLASS_NAME"/>DO.createQuery(logicalDatabase);
        else</xsl:if>
            dbQuery = <xsl:value-of select="CLASS_NAME"/>DO.createQuery();
        dbQuery.query(this);
        resultSet.next();
        rowCount=resultSet.getInt("counter");
        resultSet.close();
        dbQuery.release();
        builder.close();
        resultSet = null;
        builder.setSelectClause(tempClause);
        return rowCount;
    }
    // end of WebDocWf extension for query row counter
</xsl:template>
</xsl:stylesheet>