<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:common="org.enhydra.dods.Common"
                xmlns:dodst="org.enhydra.dods.xslt.DODSTag"
                xmlns:miu="org.enhydra.dods.xslt.XSLTUtil"
                version="1.0">
    <xsl:output method="text" encoding="iso-8859-1"/>
<xsl:template name="manyToOneReferrersDO">
<xsl:if test="dodst:compareReferrerDoName(/TABLE/PACKAGE,/TABLE/CLASS_NAME,@do_name) = 'true'">
    /**
     * Get array of <xsl:value-of select="../@name"/>DO objects that refer to this DO.
     *
     * @return Array of <xsl:value-of select="../@name"/>DO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] <xsl:value-of select="dodst:getArrayDO(../@name,@name)"/> ()
    throws DataObjectException, QueryException {
        <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] ret = null;
        try {
            <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
            if (originDatabase != null)
                q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
            else</xsl:if>
                q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
<xsl:if test="/TABLE/DO_IS_OID_BASED='true'">
            q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );</xsl:if>
<xsl:if test="not(/TABLE/DO_IS_OID_BASED='true')">
            q.setQuery<xsl:value-of select="../@name"/>DO( this, QueryBuilder.EQUAL);</xsl:if>
            ret = q.getDOArray();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
        } catch (DatabaseManagerException e) {
            throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );</xsl:if>
        } catch ( NonUniqueQueryException e ) {
            throw new DataObjectException("INTERNAL ERROR: unexpected NonUniqueQueryException" );
        } finally {
            if ( null == ret )
            ret = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[ 0 ];
        }
        return ret;
    }

    /**
     * Get the single <xsl:value-of select="../@name"/>DO object
     * that refers to this DO.
     *
     * @return <xsl:value-of select="../@name"/>DO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one <xsl:value-of select="../@name"/>DO object was found.
     */
    public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO <xsl:value-of select="dodst:getSingleDO(../@name,@name)"/> ()
    throws DataObjectException, QueryException, NonUniqueQueryException {
        <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
        try {
            if (originDatabase != null)
                q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
            else</xsl:if>
                q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
<xsl:if test="/TABLE/DO_IS_OID_BASED='true'">
            q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );</xsl:if>
<xsl:if test="not(/TABLE/DO_IS_OID_BASED='true')">
            q.setQuery<xsl:value-of select="../@name"/>DO( this, QueryBuilder.EQUAL );</xsl:if>
            q.requireUniqueInstance();
            return q.getNextDO();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
        } catch (DatabaseManagerException e) {
            throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
        }</xsl:if>
    }
<xsl:if test="/TABLE/DO_IS_OID_BASED='true'">
    /**
     * Add (set &amp; commit) a <xsl:value-of select="../@name"/>DO object that refers to this DO.
     *
     * @param referrer <xsl:value-of select="../@name"/>DO to be set to point to this DO and committed.
     *
     * @exception DatabaseManagerException if could not create a transaction.
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void <xsl:value-of select="dodst:addSingleDO(../@name,@name)"/>( <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        // WebDocWf extension for DODS row instance security
        // The following line has been changed:
        <xsl:value-of select="dodst:addSingleDO(../@name,@name)"/>( referrer, (DBTransaction)null );
        // before: <xsl:value-of select="dodst:addSingleDO(../@name,@name)"/>( referrer, null );
        // end of WebDocWf extension for DODS row instance security
    }

    /**
     * Add (set &amp; commit) a <xsl:value-of select="../@name"/>DO object that refers to this DO.
     *
     * @param referrer <xsl:value-of select="../@name"/>DO to be set to point to this DO and committed.
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction.
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void <xsl:value-of select="dodst:addSingleDO(../@name,@name)"/>( <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        referrer.set<xsl:value-of select="common:capitalizeName(@name)"/>( this );
        referrer.commit( tran );
    }

    /**
     * Remove (delete) a <xsl:value-of select="../@name"/>DO object that refers to this DO.
     *
     * @param referrer <xsl:value-of select="../@name"/>DO to be deleted.
     *
     * @exception DatabaseManagerException if could not create a transaction.
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void <xsl:value-of select="dodst:removeSingleDO(../@name,@name)"/>( <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        // WebDocWf extension for DODS row instance security
        // The following line has been changed:
        <xsl:value-of select="dodst:removeSingleDO(../@name,@name)"/>( referrer, (DBTransaction)null );
        // before: <xsl:value-of select="dodst:removeSingleDO(../@name,@name)"/>( referrer, null );
        // end of WebDocWf extension for DODS row instance security
    }

    /**
     * Remove (delete) a <xsl:value-of select="../@name"/>DO object that refers to this DO.
     *
     * @param referrer <xsl:value-of select="../@name"/>DO to be deleted.
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction.
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void <xsl:value-of select="dodst:removeSingleDO(../@name,@name)"/>( <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        <xsl:value-of select="/TABLE/CLASS_NAME"/>DO referred = referrer.get<xsl:value-of select="common:capitalizeName(@name)"/>();
        String referredHandle = referred.getHandle();
        String mydoHandle = this.getHandle();
        if ( null == referredHandle || null == mydoHandle || ( ! referredHandle.equals( mydoHandle ) ) ) {
            throw new DataObjectException( "Object " + referrer +
            " does not refer to object " + this + ", cannot be removed this way." );
        }
        referrer.delete( tran );
    }
</xsl:if>

    // WebDocWF extension for DODS row counter
    // The following lines have been added:
    /**
     * Get the number of <xsl:value-of select="../@name"/>DOs that refer to this DO.
     * via <xsl:value-of select="common:capitalizeName(@name)"/>
     *
     * @return The number of objects that refer to this DO.
     *
     * @exception DataObjectException If object is not found in the database.
     *
     * This is a WebDocWf extension for DODS row instance security.
     *
     */
    public int <xsl:value-of select="dodst:getArrayDO(../@name,@name)"/>Count ()
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException {
        int ret = 0;
        <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
        try {
            if (originDatabase != null)
                q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
            else</xsl:if>
                q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
        } catch ( DatabaseManagerException e ) {
            throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
        }</xsl:if>
        q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );
        ret = q.getCount();
        return ret;
    }
</xsl:if>
</xsl:template>
</xsl:stylesheet>


