?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:common="org.enhydra.dods.Common"
                version="1.0">
	<xsl:output method="text" encoding="iso-8859-1"/>
	<xsl:template name="getRefererrersDO">
    /**
     * Get array of <xsl:value-of select="../@name"/>DO objects that refer to this DO.
     *
     * @return Array of <xsl:value-of select="../@name"/>DO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] <xsl:value-of select="dodst:getArrayDO(../@name,@name)"/> () 
    throws DataObjectException, QueryException {
	<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] ret = null;
		try {
		    <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
   			if (originDatabase != null)
			    q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
			else 
</xsl:if>
			    q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
		    q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );
		    ret = q.getDOArray();
		} catch ( NonUniqueQueryException e ) { 
		    throw new DataObjectException( "INTERNAL ERROR: unexpected NonUniqueQueryException" );
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		} catch (DatabaseManagerException e) {
		    throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
</xsl:if>
		} finally {
		    if ( null == ret )
			ret = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[ 0 ];
		}
		return ret;
    }

    /**
     * Get single <xsl:value-of select="../@name"/>DO object that refers to this DO.
     *
     * @return <xsl:value-of select="../@name"/>DO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one <xsl:value-of select="../@name"/>DO object was found.
     */
    public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO <xsl:value-of select="dodst:getSingleDO(../@name,@name)"/> () 
    throws DataObjectException, NonUniqueQueryException, QueryException {
	<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		try {
			if (originDatabase != null)
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
			else 
</xsl:if>
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		} catch (DatabaseManagerException e) {
		    throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
		}
</xsl:if>
		q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );
		q.requireUniqueInstance();
		return q.getNextDO();
    }

	/**
	 * Get array of <xsl:value-of select="../@name"/>DO objects that refer to this DO.
	 *
	 * @return Array of <xsl:value-of select="../@name"/>DO objects.
	 *
	 * @exception DataObjectException
	 *   If the object is not found in the database.
	 * @exception QueryException
	 *   If an error occured while building the query before execution.
	 */
	public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] <xsl:value-of select="dodst:getArrayDO(../@name,@name)"/> () 
	throws DataObjectException, QueryException {
		<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] ret = null;
		<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		try {
			if (originDatabase != null)
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
			else
</xsl:if>
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
			q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );
			ret = q.getDOArray();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		} catch ( DatabaseManagerException e ) { 
		    throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
		} finally {
</xsl:if>
			if ( null == ret )
			ret = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[ 0 ];
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		}
</xsl:if>
		return ret;
	}

	/**
	 * Get single <xsl:value-of select="../@name"/>DO object that refers to this DO.
	 *
	 * @return <xsl:value-of select="../@name"/>DO object.
	 *
	 * @exception DataObjectException
	 *   If the object is not found in the database.
	 * @exception QueryException
	 *   If an error occured while building the query before execution.
	 * @exception NonUniqueQueryException
	 *   If more than one <xsl:value-of select="../@name"/>DO object was found.
	 */
	public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO <xsl:value-of select="dodst:getSingleDO(../@name,@name)"/> () 
	throws DataObjectException, NonUniqueQueryException, QueryException {
		<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		try {
			if (originDatabase != null)
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query(originDatabase);
			else
</xsl:if>
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		} catch ( DatabaseManagerException e ) { 
		    throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
		}
</xsl:if>
		q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, QueryBuilder.EQUAL );
		q.requireUniqueInstance();
		return q.getNextDO();
	}
<xsl:if test="(/TABLE/TEMPLATE_SET='webdocwf' or /TABLE/TEMPLATE_SET='multidb_webdocwf') and /TABLE/DO_IS_OID_BASED='true'">
	// WebDocWF extension for DODS row instance security
	// The following lines have been added:
	/**
	 * Get array of <xsl:value-of select="../@name"/>DO objects that refer to this DO.
	 * Only objects that can be found by the given user are returned.
	 *
	 * @param usr The user for security checks
	 *
	 * @return Array of <xsl:value-of select="../@name"/>DO objects.
	 *
	 * @exception DataObjectException
	 *   If the object is not found in the database.
	 * @exception QueryException
	 *   If an error occured while building the query before execution.
	 * @exception AccessException
	 *   The user is not allowed to create a query for 
	 *   <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/> or 
	 *   to query for the foreign key attribute <xsl:value-of select="common:capitalizeName(@name)"/>.
	 *
	 * This is a WebDocWf extension for DODS row instance security
	 *
	 */
	public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] <xsl:value-of select="dodst:getArrayDO(../@name,@name)"/> ( org.webdocwf.dods.access.User usr ) 
	throws DataObjectException, QueryException, AccessException {
		<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[] ret = null;
		<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
		try {
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
			if (originDatabase != null)
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query( originDatabase, usr );
			else
</xsl:if>
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query( usr );
			q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, usr );
			ret = q.getDOArray();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		} catch ( DatabaseManagerException e ) { 
		    throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
</xsl:if>
		} catch ( NonUniqueQueryException e ) { 
			throw new DataObjectException( "INTERNAL ERROR: unexpected NonUniqueQueryException" );
		} finally {
			if ( null == ret )
				ret = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO[ 0 ];
		}
		return ret;
	}
</xsl:if>

<xsl:if test="(/TABLE/TEMPLATE_SET='webdocwf' or /TABLE/TEMPLATE_SET='multidb_webdocwf') and /TABLE/DO_IS_OID_BASED='true'">
	/**
	 * Get single <xsl:value-of select="../@name"/>DO object that refers to this DO.
	 * The given user must have access to the DO.
	 *
	 * @param usr The user for security checks
	 *
	 * @return <xsl:value-of select="../@name"/>DO object.
	 *
	 * @exception DataObjectException
	 *   If the object is not found in the database.
	 * @exception QueryException
	 *   If an error occured while building the query before execution.
	 * @exception NonUniqueQueryException
	 *   If more than one <xsl:value-of select="../@name"/>DO object was found.
	 * @exception AccessException
	 *   The user is not allowed to create a query for 
	 *   <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/> or 
	 *   to query for the foreign key attribute <xsl:value-of select="common:capitalizeName(@name)"/> or
	 *   to read the found instance.
	 *
	 * This is a WebDocWf extension for DODS row instance security
	 *
	 */
	public <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>DO <xsl:value-of select="dodst:getSingleDO(../@name,@name)"/> ( org.webdocwf.dods.access.User usr ) 
	throws DataObjectException, NonUniqueQueryException, QueryException, AccessException {
		<xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query q;
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		try {
			if (originDatabase != null)
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query( originDatabase,usr );
			else
</xsl:if>
				q = new <xsl:value-of select="miu:getAdjustedPackageName(../@package)"/>.<xsl:value-of select="../@name"/>Query( usr );
			q.setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( this, usr );
			q.requireUniqueInstance();
			return q.getNextDO();
<xsl:if test="/TABLE/TEMPLATE_SET='multidb' or /TABLE/TEMPLATE_SET='multidb_webdocwf'">
		} catch ( DatabaseManagerException e ) { 
		    throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
		}
</xsl:if>
	}
//	end of part two of WebDocWf extension for DODS row instance security
</xsl:if>
    
	</xsl:template>
</xsl:stylesheet>

