/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: DiscFactory.java,v 1.1 2003/04/14 12:25:25 sinisa Exp $
 */

package discRack.business.disc;

import discRack.business.DiscRackBusinessException;
import discRack.data.disc.*;
import com.lutris.dods.builder.generator.query.*;
import discRack.business.person.*;
import discRack.data.person.PersonDO;
import com.lutris.appserver.server.sql.ObjectId;

import java.util.*;

/**
 * Used to find the instances of disc.
 */
public class DiscFactory {

   /*private static Map personToDiscs=new HashMap();
   private static Map IdToDisc=new HashMap();

    static {
       fillData();
    }

   public static void addPD (Person p,Disc d) {
      try {
         List l=(List)personToDiscs.get(p);
         if (l==null) {
            l=new ArrayList();
            l.add(d);
            personToDiscs.put(p,l);
         } else {
            l.add(d);
         }
         IdToDisc.put(d.getHandle(),d);
      } catch (Exception ex) {}
    }

    public static void removePD (Person p,Disc d) {
       try {
          ((List)personToDiscs.get(p)).remove(d);
          IdToDisc.remove(d.getHandle());
       } catch (Exception ex) {}
    }

    private static void fillData () {
       try {
          List discsForPerson=new ArrayList();

          Person p=PersonFactory.findPerson("sasa");
          Disc d=new Disc();
          d.setOwner(p);
          d.setTitle("album 1");
          d.setArtist("U2");
          d.setGenre("Rock");
          d.setLiked(true);
          discsForPerson.add(d);
          IdToDisc.put(d.getHandle(),d);

          d=new Disc();
          d.setOwner(p);
          d.setTitle("album 2");
          d.setArtist("U2");
          d.setGenre("Rock");
          d.setLiked(true);
          discsForPerson.add(d);
          IdToDisc.put(d.getHandle(),d);

          d=new Disc();
          d.setOwner(p);
          d.setTitle("album 3");
          d.setArtist("U2");
          d.setGenre("Rock");
          d.setLiked(false);
          discsForPerson.add(d);
          IdToDisc.put(d.getHandle(),d);

          personToDiscs.put(p,new ArrayList(discsForPerson));

          discsForPerson.clear();
          p=PersonFactory.findPerson("dragan");
          d=new Disc();
          d.setOwner(p);
          d.setTitle("album 1");
          d.setArtist("Azra");
          d.setGenre("Rock");
          d.setLiked(true);
          discsForPerson.add(d);
          IdToDisc.put(d.getHandle(),d);

          d=new Disc();
          d.setOwner(p);
          d.setTitle("album 2");
          d.setArtist("Azra");
          d.setGenre("Rock");
          d.setLiked(false);
          discsForPerson.add(d);
          IdToDisc.put(d.getHandle(),d);

          d=new Disc();
          d.setOwner(p);
          d.setTitle("album 3");
          d.setArtist("Azra");
          d.setGenre("Rock");
          d.setLiked(true);
          discsForPerson.add(d);
          IdToDisc.put(d.getHandle(),d);

          personToDiscs.put(p,new ArrayList(discsForPerson));


          discsForPerson.clear();
          p=PersonFactory.findPerson("neso");
          d=new Disc();
          d.setOwner(p);
          d.setTitle("album 1");
          d.setArtist("EKV");
          d.setGenre("Rock");
          d.setLiked(true);
          discsForPerson.add(d);
          IdToDisc.put(d.getHandle(),d);

          d=new Disc();
          d.setOwner(p);
          d.setTitle("album 2");
          d.setArtist("EKV");
          d.setGenre("Rock");
          d.setLiked(false);
          discsForPerson.add(d);
          IdToDisc.put(d.getHandle(),d);

          d=new Disc();
          d.setOwner(p);
          d.setTitle("album 3");
          d.setArtist("EKV");
          d.setGenre("Rock");
          d.setLiked(true);
          discsForPerson.add(d);
          IdToDisc.put(d.getHandle(),d);

          personToDiscs.put(p,discsForPerson);
       } catch (Exception ex) {}
   }*/

   /**
    * The findDiscsForPerson method performs a database query to
    * return an array of <CODE>Disc</CODE> objects
    * representing all the rows in the <CODE>disc</CODE> table
    * that have and owner matching <CODE>Person owner</CODE>.
    *
    * @param owner. The owner of the discs
    * @return
    *     array of discs.
    * @exception DiscRackBusinessException
    *   If there is a problem retrieving disc information.
    */
   public static Disc[] findDiscsForPerson(Person owner)
   throws DiscRackBusinessException {
      Disc[] theDiscArray = null;

      try {
         DiscQuery query = new DiscQuery();
         //set query
         query.setQueryOwner(PersonDO.createExisting(owner.getHandle()));
         // Order discs alphabetically by artist
         query.addOrderByArtist();
         DiscDO[] DOarray = query.getDOArray();
         theDiscArray = new Disc[ DOarray.length ];
         for ( int i = 0; i < DOarray.length; i++ )
            theDiscArray[i] = new Disc(DOarray[i]);
      }catch(Exception ex) {
         throw new DiscRackBusinessException("Exception in findDiscsForPerson()", ex);
      }
      return theDiscArray;
      /*
      List l=(List)personToDiscs.get(owner);
      Disc[] tda=new Disc[l.size()];
      l.toArray(tda);
      return tda;
      */
   }

   /**
    * The findDiscByID method performs a database query to
    * return a <CODE>Disc</CODE> object
    * representing the row in the <CODE>disc</CODE> table
    * that matches the object id.
    *
    * @param id, the object id of the disc table.
    * @return
    *    the disc. null if there isn't a person associated
    *    the id
    * @exception DiscRackBusinessException
    *    if there is a problem retrieving person information.
    */
   public static Disc findDiscByID(String id)
   throws DiscRackBusinessException {
      Disc theDisc = null;

      try {
         DiscQuery query = new DiscQuery();
         //set query
         query.setQueryOId(new ObjectId(id));
         // Throw an exception if more than one user by this name is found
         query.requireUniqueInstance();
         DiscDO theDiscDO = query.getNextDO();
         theDisc = new Disc(theDiscDO);
         return theDisc;
      }catch(Exception ex) {
         throw new DiscRackBusinessException("Exception in findDiscsForPerson()", ex);
      }
      //return (Disc)IdToDisc.get(id);
   }
}

