/*-----------------------------------------------------------------------------
 *  PersonQuery.java
 *-----------------------------------------------------------------------------
 */

package phoneList.data;

import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.Enhydra;
import java.sql.*;
import java.util.*;
import java.math.BigDecimal;
import com.lutris.appserver.server.sql.ObjectIdException;

/**
 * @author Kristen Pol
 * @since Enhydra2.0
 */
public class PersonQuery implements Query {

    // Construct new person query object
    public PersonQuery() {
    }

    // Object id for query
    private ObjectId queryOId = null;

    // Id for query
    private String queryId = null;

    // First name for query
    private String queryFirstName = null;

    // Last name for query
    private String queryLastName = null;

    // Phone number for query
    private String queryPhoneNumber = null;

    // Keep track of which query fields are set
    private BitSet query = new BitSet();

    // Keep track if we are querying by object id
    private final int QUERY_OID = 1;

    // Keep track if we are querying id
    private final int QUERY_ID = 2;

    // Keep track if we are querying first name
    private final int QUERY_FIRST_NAME = 3;

    // Keep track if we are querying last name
    private final int QUERY_LAST_NAME = 4;

    // Keep track if we are querying phone number
    private final int QUERY_PHONE_NUMBER = 5;

    // Execute the query
    // Either query by OId or by a specific parameter; or get entire table
    public ResultSet executeQuery(DBConnection conn)
	    throws SQLException {
	int param = 1;
        PreparedStatement stmt;
        if (query.get(QUERY_OID)) {
            if (queryOId != null) {
		String sql = "select * from " + PersonDO.getDBTableName() + 
			" where" + " oId = ?";
                stmt = conn.prepareStatement(sql);
                stmt.setBigDecimal(param++, queryOId.toBigDecimal());
            } else {
                return null;
            }
        } else if (query.get(QUERY_ID) || query.get(QUERY_FIRST_NAME) ||
                 query.get(QUERY_LAST_NAME) || query.get(QUERY_PHONE_NUMBER)) {
            StringBuffer sql = new StringBuffer(
		"select * from " + PersonDO.getDBTableName() + " where");

	    // Set up which parameters to query for
            boolean and = false;
	    and = addQuery(sql, QUERY_FIRST_NAME, queryFirstName, 
		PersonDO.FIRST_NAME, and);
	    and = addQuery(sql, QUERY_LAST_NAME, queryLastName, 
		PersonDO.LAST_NAME, and);
	    and = addQuery(sql, QUERY_PHONE_NUMBER, queryPhoneNumber, 
		PersonDO.PHONE_NUMBER, and);

	    // Set up the values of these query parameters
            stmt = conn.prepareStatement(sql.toString());
	    param = setString(stmt, param, QUERY_FIRST_NAME, queryFirstName);
	    param = setString(stmt, param, QUERY_LAST_NAME, queryLastName);
	    param = setString(stmt, param, QUERY_PHONE_NUMBER, queryPhoneNumber);
        } else {
            // Get entire table
            stmt = conn.prepareStatement(
		"select * from " + PersonDO.getDBTableName());
        }
        return conn.executeQuery(stmt, "Person query");
    }

    // Get next PersonDO object
    public Object next(ResultSet rs)
	    throws SQLException, ObjectIdException {
        if (! rs.next())
            return null;
        return new PersonDO(rs);
    }

    // Set OId query bit
    public void setQueryOId(ObjectId oId) {
        this.queryOId = oId;
        query.set(QUERY_OID);
    }


    // Set FirstName query bit
    public void setQueryFirstName(String firstName) {
        this.queryFirstName = firstName;
        query.set(QUERY_FIRST_NAME);
    }

    // Set LastName query bit
    public void setQueryLastName(String lastName) {
        this.queryLastName = lastName;
        query.set(QUERY_LAST_NAME);
    }

    // Set PhoneNumber query bit
    public void setQueryPhoneNumber(String phoneNumber) {
        this.queryPhoneNumber = phoneNumber;
        query.set(QUERY_PHONE_NUMBER);
    }

    // Clear all query bits
    public void reset() {
        query.clear(QUERY_OID);
        query.clear(QUERY_ID);
        query.clear(QUERY_FIRST_NAME);
        query.clear(QUERY_LAST_NAME);
        query.clear(QUERY_PHONE_NUMBER);
    }

    // Add a query to the sql
    public boolean addQuery(StringBuffer sql, int queryBit, 
	    Object queryParam, String paramName, boolean and) {
	if (query.get(queryBit)) {
	    if (and) sql.append(" and");
	    if (queryParam != null) {
		sql.append(paramName + " = ?");
	    } else {
		sql.append(paramName + " is null");
	    }
	    and = true;
	}
	return and;
    }

    // Add a query to the sql
    public int setString(PreparedStatement stmt, int param, int queryBit, 
	    String queryParam) 
	    throws SQLException {
	if (query.get(queryBit)) {
	    if (queryParam != null) {
		stmt.setString(param++, queryParam);
	    }
	}
	return param;
    }

    public static void testPersonQuery() {
        try {
            PersonQuery personQuery = new PersonQuery();
            DBQuery dbQuery = Enhydra.getDatabaseManager().createQuery();
            try {
                dbQuery.query(personQuery);
                PersonDO personDO = null;
                while ((personDO = (PersonDO) dbQuery.next()) != null) {
		    System.out.println("PersonDO: first name = " + 
			personDO.getFirstName());
		    System.out.println("PersonDO: last name = " + 
			personDO.getLastName());
		    System.out.println("PersonDO: phone number = " + 
			personDO.getPhoneNumber());
                }
            } finally {
                dbQuery.release();
            }
        } catch (Exception e) {
	    System.out.println("PersonQuery error: " + e.getMessage());
            e.printStackTrace();
        }
    }
}
