/*
 * put your module comment here
 * formatted with JxBeauty (c) johann.langhofer@nextra.at
 */


package  phoneList.business;

import  java.util.Vector;
import  java.io.*;
import  java.util.*;
import  java.net.*;
import  org.w3c.dom.*;
import  org.apache.soap.util.xml.*;
import  org.apache.soap.*;
import  org.apache.soap.encoding.*;
import  org.apache.soap.encoding.soapenc.*;
import  org.apache.soap.rpc.*;
import org.apache.log4j.Logger;

/**
 * PhoneList defines a set of static methods that are used
 * to query and update phone information in the database.
 */
public class PhoneList {
  public static String server = "localhost";
  public static String port = "8080";

  public static String setServerUrl (String s, String p) {
    server = s;
    port = p;
    return  "http://" + server + ":" + port + "/soap/servlet/rpcrouter";
  }
  public static String surl = setServerUrl(server, port);

  /**
   * Queries the database for all people/phones.
   *
   * @return Vector[0] contains the people's first names.
   *   Vector[1] contains the people's last names.
   *   Vector[2] contains the phone numbers.
   *   Vector[3] contains the people's identifiers.
   * @exception Exception all exceptions are thrown whenever an
   *   error occurs.
   */
  public static Vector[] getLists () throws Exception {
    Vector[] results = new Vector[4];
    results[0] = new Vector();
    results[1] = new Vector();
    results[2] = new Vector();
    results[3] = new Vector();
    String encodingStyleURI = Constants.NS_URI_LITERAL_XML;
    URL url = new URL(surl);
    SOAPMappingRegistry smr = new SOAPMappingRegistry();
    BeanSerializer beanSer = new BeanSerializer();
    Call call = new Call();
    call.setSOAPMappingRegistry(smr);
    call.setTargetObjectURI("urn:PhoneBook");
    call.setMethodName("getAllEntries");
    call.setEncodingStyleURI(encodingStyleURI);
    // Invoke the call.
    Response resp;
    try {
      resp = call.invoke(url, "");
    } catch (SOAPException e) {
      System.err.println("Caught SOAPException (" + e.getFaultCode() + "): "
          + e.getMessage());
      throw  e;
    }
    // Check the response.
    if (!resp.generatedFault()) {
      Parameter ret = resp.getReturnValue();
      Element bookEl = (Element)ret.getValue();
      Logger.getLogger("org.enhydra.PhoneBookSoapClient").debug("RESPONSE - Phonebook:");
      Logger.getLogger("org.enhydra.PhoneBookSoapClient").debug(DOM2Writer.nodeToString(bookEl));
      NodeList nodeFirstName = bookEl.getElementsByTagName("FirstName");
      NodeList nodeLastName = bookEl.getElementsByTagName("LastName");
      NodeList nodePhoneNumber = bookEl.getElementsByTagName("PhoneNumber");
      NodeList nodeId = bookEl.getElementsByTagName("Id");
      for (int i = 0; i < nodeFirstName.getLength(); i++) {
        results[0].addElement(((Element)nodeFirstName.item(i)).getAttribute("value"));
        results[1].addElement(((Element)nodeLastName.item(i)).getAttribute("value"));
        results[2].addElement(((Element)nodePhoneNumber.item(i)).getAttribute("value"));
        results[3].addElement(((Element)nodeId.item(i)).getAttribute("value"));
      }
    }
    else {
      Fault fault = resp.getFault();
      System.err.println("Generated fault: " + fault);
    }
    return  results;
  }

  /**
   * Creates a new person and adds him/her to the database.
   * Demonstrates how to use a
   * transaction to insert data into the database.
   *
   * @param firstName the person's first name.
   * @param lastName the person's last name.
   * @param phoneNumber the person's phone number.
   *
   * @exception Exception all exceptions are thrown.  A more
   *   robust application would handle them.....
   */
  public static void addPerson (String firstName, String lastName, String phoneNumber) throws Exception {
    URL url = new URL(surl);
    SOAPMappingRegistry smr = new SOAPMappingRegistry();
    BeanSerializer beanSer = new BeanSerializer();
    Call call = new Call();
    call.setSOAPMappingRegistry(smr);
    call.setTargetObjectURI("urn:PhoneBook");
    call.setMethodName("addEntry");
    call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
    Vector params = new Vector();
    params.addElement(new Parameter("firstName", String.class, firstName, null));
    params.addElement(new Parameter("lastName", String.class, lastName, null));
    params.addElement(new Parameter("phoneNumber", String.class, phoneNumber,
        null));
    call.setParams(params);
    // Invoke the call.
    Response resp;
    try {
      resp = call.invoke(url, "");
    } catch (SOAPException e) {
      System.err.println("Caught SOAPException (" + e.getFaultCode() + "): "
          + e.getMessage());
      return;
    }
    // Check the response.
    if (!resp.generatedFault()) {
      Logger.getLogger("org.enhydra.PhoneBookSoapClient").debug(firstName + " " + lastName + " has been added.");
    }
    else {
      Fault fault = resp.getFault();
      System.err.println("Generated fault: " + fault);
    }
  }

  /**
   * Deletes a person from the database.  Demonstrates how to use a
   * transaction to delete data in the database.
   *
   * @param id the person's id
   *
   * @exception Exception all exceptions are thrown.  A more
   *   robust application would handle them.....
   */
  public static void deletePerson (String id) throws Exception {
    URL url = new URL(surl);
    SOAPMappingRegistry smr = new SOAPMappingRegistry();
    BeanSerializer beanSer = new BeanSerializer();
    Call call = new Call();
    call.setSOAPMappingRegistry(smr);
    call.setTargetObjectURI("urn:PhoneBook");
    call.setMethodName("deleteEntry");
    call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
    Vector params = new Vector();
    params.addElement(new Parameter("id", String.class, id, null));
    call.setParams(params);
    // Invoke the call.
    Response resp;
    try {
      resp = call.invoke(url, "");
    } catch (SOAPException e) {
      System.err.println("Caught SOAPException (" + e.getFaultCode() + "): "
          + e.getMessage());
      return;
    }
    // Check the response.
    if (!resp.generatedFault()) {
      Logger.getLogger("org.enhydra.PhoneBookSoapClient").debug(id + " has been deleted.");
    }
    else {
      Fault fault = resp.getFault();
      System.err.println("Generated fault: " + fault);
    }
  }

  /**
   * Modifies a person's profile.  Demonstrates how to use a
   * transaction to update data in the database.
   *
   * @param id the person's id
   * @param newFirstName the person's new first name.
   * @param newLastName the person's new last name.
   * @param newPhoneNumber the person's new phone number.
   *
   * @exception Exception all exceptions are thrown.  A more
   *   robust application would handle them.....
   */
  public static void modifyPerson (String id, String newFirstName, String newLastName,
      String newPhoneNumber) throws Exception {
    URL url = new URL(surl);
    SOAPMappingRegistry smr = new SOAPMappingRegistry();
    BeanSerializer beanSer = new BeanSerializer();
    Call call = new Call();
    call.setSOAPMappingRegistry(smr);
    call.setTargetObjectURI("urn:PhoneBook");
    call.setMethodName("modifyEntry");
    call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
    Vector params = new Vector();
    params.addElement(new Parameter("id", String.class, id, null));
    params.addElement(new Parameter("firstName", String.class, newFirstName,
        null));
    params.addElement(new Parameter("lastName", String.class, newLastName,
        null));
    params.addElement(new Parameter("phoneNumber", String.class, newPhoneNumber,
        null));
    call.setParams(params);
    // Invoke the call.
    Response resp;
    try {
      resp = call.invoke(url, "");
    } catch (SOAPException e) {
      System.err.println("Caught SOAPException (" + e.getFaultCode() + "): "
          + e.getMessage());
      return;
    }
    // Check the response.
    if (!resp.generatedFault()) {
      Logger.getLogger("org.enhydra.PhoneBookSoapClient").debug(newFirstName + " " + newLastName + " has been modified.");
    }
    else {
      Fault fault = resp.getFault();
      System.err.println("Generated fault: " + fault);
    }
  }

  /**
   * Queries a person from the database.
   *
   * @param id the person's id.
   * @exception Exception all exceptions are thrown.  A more
   *   robust applications would handle them.....
   */
  public static Person getById (String id) throws Exception {
    String encodingStyleURI = Constants.NS_URI_SOAP_ENC;
    URL url = new URL(surl);
    SOAPMappingRegistry smr = new SOAPMappingRegistry();
    BeanSerializer beanSer = new BeanSerializer();
    // Map the types.
    smr.mapTypes(Constants.NS_URI_SOAP_ENC, new QName("urn:xml-soap-phoneBook",
        "person"), Person.class, beanSer, beanSer);
    // Build the call.
    Call call = new Call();
    call.setSOAPMappingRegistry(smr);
    call.setTargetObjectURI("urn:PhoneBook");
    call.setMethodName("getById");
    call.setEncodingStyleURI(encodingStyleURI);
    Vector params = new Vector();
    params.addElement(new Parameter("id", String.class, id, null));
    call.setParams(params);
    // Invoke the call.
    Response resp;
    try {
      resp = call.invoke(url, "");
    } catch (SOAPException e) {
      System.err.println("Caught SOAPException (" + e.getFaultCode() + "): "
          + e.getMessage());
      throw  e;
    }
    // Check the response.
    if (!resp.generatedFault()) {
      Parameter ret = resp.getReturnValue();
      Person value = (Person)ret.getValue();
      if (value == null) {
        return  null;
      }
      return  value;
    }
    else {
      Fault fault = resp.getFault();
      System.err.println("Generated fault: " + fault);
      return  null;
    }
  }
}



