/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 */

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * <p>
 *  <code>CommandLineErrorHandler</code> is a simple SAX 
 *    <code>ErrorHandler</code> that prints out errors and fatal errors to
 *    the command line. Warnings are ignored.
 * </p>
 */
public class CommandLineErrorHandler implements ErrorHandler {

    /**
     * <p>
     *  Indicates a warning has occurred.
     * </p>
     *
     * @param e the parsing exception that triggered this warning.
     * @throws <code>SAXException</code> - allows for the exception to be
     *         re-thrown.
     */
    public void warning(SAXParseException e) throws SAXException {
        // No action... warnings are OK
    }
    
    /**
     * <p>
     *  Indicates an error has occurred.
     * </p>
     *
     * @param e the parsing exception that triggered this error.
     * @throws <code>SAXException</code> - allows for the exception to be
     *         re-thrown.
     */
    public void error(SAXParseException e) throws SAXException {
        System.out.println("Error occurred: " + e.getMessage());
        throw e;
    }
    
    /**
     * <p>
     *  Indicates a fatal error has occurred.
     * </p>
     *
     * @param e the parsing exception that triggered this fatal error.
     * @throws <code>SAXException</code> - allows for the exception to be
     *         re-thrown.
     */
    public void fatalError(SAXParseException e) throws SAXException {
        System.out.println("Fatal error occurred: " + e.getMessage());
        throw e;
    }

}
