/*
 * LoaderTest.java   Sept 01, 2002
 *
 * Sample JUnit test using Loader for creating test database and 
 * inserting data into it.
 *
 */


import java.sql.Connection;
import java.sql.DriverManager;

import org.webdocwf.util.loader.test.LoaderTestCase;
import org.webdocwf.util.loader.test.DatabaseOperation;
import org.webdocwf.util.loader.test.CreateDatabaseOperation;
import org.webdocwf.util.loader.test.DropDatabaseOperation;
import org.webdocwf.util.loader.test.LoaderOperation;
import org.webdocwf.util.loader.Loader;

import junit.framework.TestCase;
import junit.framework.Test;
import junit.framework.TestSuite;
import junit.framework.TestResult;

/**
 * @author Sinisa Milosevic
 * @version $Revision: 1.1 $
 */
public class LoaderTest3 extends LoaderTestCase
{
   
    public LoaderTest3(String name)
    {
        super(name);
    }

    /**
     * Returns the test database connection.
     */
    public Connection getConnection() throws Exception
    	{
    	
  		 	Class driverClass = Class.forName("com.internetcds.jdbc.tds.Driver");
       	Connection jdbcConnection = DriverManager.getConnection(
                "jdbc:freetds:sqlserver://192.168.0.4:1433/master", "sa", "");

    		return jdbcConnection;	
    	}	

     /**
     * Returns the name of test database.
     */

    public String getDatabaseName() throws Exception
		{
			return "OMDSTest3";	
		}

   
    /**
     * Returns the test Loader class (loaderjob).
     */
    public Loader getLoader() throws Exception
    	{
    		Loader loadJob= new Loader("../examples/ObjectLoader/CreateTables.xml");
    		loadJob.setUserID("admin");
    		
    		return loadJob;
    	}

    /**
     * Returns the database operations executed in test setup. First operation will be 
     * executed dbOperation[0], then dbOperation[1]...
     */
    public DatabaseOperation[] getSetUpOperation() throws Exception
    {
//        Creating test database.....	
        DatabaseOperation[] dbOperation = new DatabaseOperation[4];
        dbOperation[0]=new CreateDatabaseOperation(getDatabaseName());
        
//        Creating tables.....	
        dbOperation[1]=new LoaderOperation(getLoader());
        
//		    Inserting data.....	
		  Loader loadJob1= new Loader("../examples/ObjectLoader/InsertData.xml");
    	  loadJob1.setUserID("admin");
		  dbOperation[2]=new LoaderOperation(loadJob1);

//		    Creating indexes, foreign keys.....	
		  Loader loadJob2= new Loader("../examples/ObjectLoader/CreateIndex.xml");
		  dbOperation[3]=new LoaderOperation(loadJob2);
        
        return dbOperation;
    }
    
        /**
     * Returns the database operation executed in test cleanup.
     *  First operation will be executed dbOperation[0], then dbOperation[1]...
     */
    public DatabaseOperation[] getTearDownOperation() throws Exception
    {
//		    Deleting test database.....	
        DatabaseOperation[] dbOperation = new DatabaseOperation[1];
        dbOperation[0]=new DropDatabaseOperation(getDatabaseName());

        return dbOperation;
    }


	public void testMe() throws Exception
    {
		System.out.println("Executing test: test1");
	}
	

   public static Test suite() { 
    return new TestSuite(LoaderTest3.class); 
}
 	
    	
   public static void main(String args[]) { 
//    	junit.textui.TestRunner.run(suite());
	TestResult result= (new LoaderTest3("testMe")).run();    	

}



}





