
/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * org.enhydra.barracuda.discRack.data.disc/DiscDOI.java
 *-----------------------------------------------------------------------------
 */

package org.enhydra.barracuda.discRack.data.disc;

import java.io.*;
import java.sql.*;
import java.math.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.dods.builder.generator.query.*;

/**
 * Interface implemented by DiscDO
 * Interface could also be implemented by a (hand-written) Business Layer class
 * which uses DiscDO
 *
 * @version $Revision: 1.1 $
 * @author  NN
 * @since   Enhydra Project
 */
public interface DiscDOI {

    ////////////////////////// data member Title

    /**
     * Get title of the Disc
     *
     * @return title of the Disc
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public String getTitle () 
    throws DataObjectException;
    

    /**
     * Set title of the Disc
     *
     * @param title of the Disc
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void setTitle ( String title )
    throws DataObjectException;

    ////////////////////////// data member Artist

    /**
     * Get artist of the Disc
     *
     * @return artist of the Disc
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public String getArtist () 
    throws DataObjectException;
    

    /**
     * Set artist of the Disc
     *
     * @param artist of the Disc
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void setArtist ( String artist )
    throws DataObjectException;

    ////////////////////////// data member Genre

    /**
     * Get genre of the Disc
     *
     * @return genre of the Disc
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public String getGenre () 
    throws DataObjectException;
    

    /**
     * Set genre of the Disc
     *
     * @param genre of the Disc
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void setGenre ( String genre )
    throws DataObjectException;

    ////////////////////////// data member Owner

    /**
     * Get owner of the Disc
     *
     * @return owner of the Disc
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public org.enhydra.barracuda.discRack.data.person.PersonDO getOwner () 
    throws DataObjectException;
    

    /**
     * Set owner of the Disc
     *
     * @param owner of the Disc
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void setOwner ( org.enhydra.barracuda.discRack.data.person.PersonDO owner )
    throws DataObjectException;

    ////////////////////////// data member IsLiked

    /**
     * Get isLiked of the Disc
     *
     * @return isLiked of the Disc
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public boolean getIsLiked () 
    throws DataObjectException;
    

    /**
     * Set isLiked of the Disc
     *
     * @param isLiked of the Disc
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void setIsLiked ( boolean isLiked )
    throws DataObjectException;

    /**
     * Inserts/Updates the DO into its table.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
     * @exception RefAssertionException thrown by okTo method.
     * @exception java.sql.SQLException if any SQL errors occur.
     */
    public void save() 
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException;
    
    /**
     * Inserts/Updates the DO into its table.
     * The transaction is likely provided by the save() method of another DO
     * which references this DO.
     * 
     * @param dbt The transaction object to use for this operation.
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
     * @exception RefAssertionException thrown by okTo method.
     * @exception java.sql.SQLException if any SQL errors occur.
     */
    public void save(DBTransaction dbt)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException;
    
    /**
     * Inserts/Updates the DO into its table.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
     * @exception RefAssertionException thrown by okTo method.
     * @exception java.sql.SQLException if any SQL errors occur.
     * @deprecated Use save() instead.
     */
    public void commit() 
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException;
    
    /**
     * Inserts/Updates the DO into its table.
     * The transaction is likely provided by the commit() method of another DO
     * which references this DO.
     * 
     * @param dbt The transaction object to use for this operation.
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
     * @exception RefAssertionException thrown by okTo method.
     * @exception java.sql.SQLException if any SQL errors occur.
     * @deprecated Use save() instead.
     */
    public void commit(DBTransaction dbt)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException;
    
    /**
     * Deletes the DO from its table.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
     * @exception RefAssertionException thrown by okTo method.
     * @exception java.sql.SQLException if any SQL errors occur.
     */
    public void delete() 
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException;
    
    /**
     * Deletes the DO from its table.
     * The transaction is likely provided by the delete() method of another DO
     * which references this DO.
     *
     * @param dbt The transaction object to use for this operation.
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
     * @exception RefAssertionException thrown by okTo method.
     * @exception java.sql.SQLException if any SQL errors occur.
     */
    public void delete(DBTransaction dbt)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException;

}
