
/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * contains WebDocWf extensions
 *
 *-----------------------------------------------------------------------------
 * org.enhydra.barracuda.discRack.data.person/PersonDO.java
 *-----------------------------------------------------------------------------
 */

package org.enhydra.barracuda.discRack.data.person;

import java.io.*;
import java.sql.*;
import java.math.*;
import java.util.Hashtable;
import java.util.Collection;
import java.util.Properties;
import java.io.FileInputStream;

import org.enhydra.dods.DODS;
import com.lutris.util.Config;
import com.lutris.util.ConfigException;
import com.lutris.appserver.server.sql.*;
import com.lutris.dods.builder.generator.dataobject.*;
import com.lutris.dods.builder.generator.query.*;
import org.enhydra.dods.cache.LRUCache;
import org.enhydra.dods.cache.Condition;
import org.enhydra.dods.cache.QueryCacheImpl;
import org.enhydra.dods.cache.QueryCache;
import org.enhydra.dods.cache.QueryResult;
import org.enhydra.dods.cache.CacheAdministration;
import org.enhydra.dods.statistics.Statistics;
import org.enhydra.dods.statistics.TableStatistics;
import org.enhydra.dods.statistics.QueryCacheStatistics;

// WebDocWf extension for DODS row instance security and genric store
import org.webdocwf.dods.access.*;
import org.webdocwf.util.genericstore.GenericStoreDO;
import java.util.Vector;
import java.util.HashSet;
import java.util.Iterator;
// end of WebDocWf extension for DODS row instance security and genric store

/**
 * Data core class, used to set and retrieve the PersonDO information.
 *
 * @version $Revision: 1.7 $
 * @author  NN
 * @since   Enhydra Project
 */
 public class PersonDO extends com.lutris.dods.builder.generator.dataobject.GenericDO implements PersonDOI, java.io.Serializable {
    /**
     * Static final data members name the table and columns for this DO.
     * By using these members with an instance of the QueryBuilder class,
     * an application can perform straight SQL queries while retaining
     * compile-time checking of table and column usage.
     *
     * Example:  List the Cities containing Persons named Bob:
     *
     *    Using straight SQL with QueryBuilder:
     *      Pro: code runs faster because you create fewer objects
     *      Con: code is less clear
     *
     *         Vector fields = new Vector();
     *         fields.addElement( AddressDO.City );
     *         QueryBuilder qb = new QueryBuilder( fields );
     *         qb.addWhere( PersonDO.FirstName, "Bob" );
     *         qb.addWhere( PersonDO.PrimaryKey, AddressDO.Person );
     *         RDBRow row;
     *         while ( null != ( row = qb.getNextRow() ) ) {
     *             String city = row.get( AddressDO.City ).getString();
     *         }
     *
     *    Using Query/DO classes:
     *      Pro: code is (often) clearer
     *      Con: code runs slower because you create more objects
     *
     *         PersonQuery pq = new PersonQuery();
     *         pq.setQueryFirstName( "Bob" );
     *         PersonDO[] bobs = pq.getDOArray();
     *         for ( int i = 0; i < bobs.length; i++ ) {
     *             AddressQuery aq = new AddressQuery();
     *             aq.setQueryPerson( bobs[i] );
     *             AddressDO addr = aq.getNextDO();
     *             String city = addr.getCity();
     *         }
     */
    static public final RDBTable  table = new RDBTable( "person" );

    /**
     * Name of the logical database for which DO object was created
     */
    protected String originDatabase = null;

    /**
     * Return the name of the logical database for which DO object was created.
     *
     * @return origin logical database name.
     *
     */
    public String getOriginDatabase() {
        return originDatabase;
    }

    /**
     * Return person as the name of the table in the database
     * which contains PersonDO objects.
     * This method overrides CoreDO.getTableName()
     * and is used by CoreDO.executeUpdate() during error handling.
     *
     * @return The database table name.
     *
     * @see CoreDO
     * @author Jay Gunter
     */
    public String getTableName() {
        return "person";
    }

    /**
     * static final RDBColumn PrimaryKey for use with QueryBuilder.
     * See example above.
     */
    static public final RDBColumn PrimaryKey = new RDBColumn( table, getPrimaryKeyName() );

    /* RDBColumns for PersonDO attributes are defined below. */


    /* Using a DO (and its Query class) to access a VIEW instead of a TABLE:
     *
     * A DO (and its Query class) can be used to access a VIEW
     * instead of a TABLE.  The Data Object is created as usual in DODS,
     * but the "create table" SQL command for that DO is not used.
     * Instead, you substitute a "create view" command to create a
     * virtual table in the database; this is often done to provide
     * convenient access to a collection of tables joined together.
     *
     * A VIEW usually does not return "oid" and "version" columns;
     * often (but now always) a VIEW is defined to return the "oid" column
     * for one of the tables joined together in the definition of the VIEW.
     *
     * If notUsingOId is true, PersonDO.createExisting(ResultSet)
     * will NOT invoke the GenericDO(ResultSet) constructor
     * so to avoid attempting to extract the "oid" and "version" columns
     * from the ResultSet.
     */
    static protected final boolean notUsingOId = false;


    /**
     * A DO class contains a reference to a DataStruct class.
     * This reference can be null (when the data for the DO
     * has not yet been retrieved from the database),
     * allowing a DO object to be a lightweight placeholder
     * until its data is needed.
     */
    private PersonDataStruct data = null;

    /**
     * Return information whether the data for this object has been marked read-only.
     *
     * @return True if the data for this object has been marked read-only.
     *
     */
    public boolean isReadOnly() {
        if ( null == data )
            return false;
        return data.readOnly;
    }

    /**
     * Sets DO's data.
     */
     public void setData (Object data) {
        this.data = (PersonDataStruct)data;
     }

    /**
     * Returns DO's data.
     */
     public Object getData () {
        return data;
     }

    /**
     * Protected constructor. Only derived classes should call it.
     *
     * @param is_view
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     */
    protected PersonDO ( boolean is_view )
    throws ObjectIdException, DatabaseManagerException {
        super( is_view );
        originDatabase = getLogicalDBName();
    }

    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     */
    protected PersonDO ()
    throws ObjectIdException, DatabaseManagerException {
        super( notUsingOId );

        originDatabase = getLogicalDBName();
    }


    /**
     * isLoaded()
     * Returns information whether object's data is loaded from database.
     * @return true if the data for this object has been retrieved
     * from the database.
     */
    public boolean isLoaded() {
        return ( null != data );
    }

    /**
     * Load the fields for the DO from the database.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public void loadData()
    throws SQLException, ObjectIdException, DataObjectException {
        if ( null == data ) {
            data = new PersonDataStruct ();
        }

        ObjectId id = getOId();
        if ( null == id )
            return;
        if ( ! isPersistent() )   // DO from createVirgin
            return;

        // DO from createExisting.  Complain if no record in database.
        PersonQuery query;

                query = new PersonQuery ();

        query.setQueryOId( id );
        query.requireUniqueInstance();
        PersonDO obj;
        try {
            obj = query.getNextDO();
            if ( null == obj )
                throw new DataObjectException("PersonDO DO not found for id=" + id );
            makeIdentical(obj);
            setVersion(    obj.getVersion() );
            setNewVersion( obj.getVersion() );
        } catch ( NonUniqueQueryException e ) {
            throw new ObjectIdException( "Duplicate ObjectId" );
        }

    }

    /**
     * Load the actual DO data if necessary.
     * Called by get/set methods.
     *
     * @exception DataObjectException If a data access error occurs.
     */
    protected void checkLoad()
    throws DataObjectException {
        if ( null == data )
            try {
                loadData();
            } catch ( Exception e ) {
                throw new DataObjectException("Unable to load data for PersonDO id=" + getOId() +
                                              ", error = " + e.getMessage() );
            }
    }

    /**
     * Protected constructor used by createExisting(ObjectId) above.
     *
     * @param id The ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   Should never see this exception since GenericDO.ctor(ObjectId)
     *   never accesses the database.
     */
    protected PersonDO( ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        super( id );
        originDatabase = getLogicalDBName();
    }


    /**
     * Represents table and cache (if there is caching) statistics.
     */
    protected static Statistics statistics;

    /**
     * Get table statistics.
     *
     * @return Table statistics.
     */
    public static Statistics getStatistics() {
        if (cache != null) {
            ((QueryCacheStatistics)statistics).setCacheType(cache.getCacheType());
            ((QueryCacheStatistics)statistics).setMaxCacheSize(cache.getMaxCacheSize(false));
            ((QueryCacheStatistics)statistics).setCacheSize(cache.getCacheSize());
            ((QueryCacheStatistics)statistics).setComplexQueryCacheSize(cache.getComplexQueryCacheSize());
            ((QueryCacheStatistics)statistics).setMaxComplexQueryCacheSize(cache.getMaxComplexQueryCacheSize(false));
            ((QueryCacheStatistics)statistics).setSimpleQueryCacheSize(cache.getSimpleQueryCacheSize());
            ((QueryCacheStatistics)statistics).setMaxSimpleQueryCacheSize(cache.getMaxSimpleQueryCacheSize(false));
            statistics.stopTime();
        }
        return statistics;
    }

    /**
     * Refresh table statistics.
     */
    public static void refreshStatistics() {
        if (cache != null)
            statistics = new QueryCacheStatistics("person");
        else
            statistics = new TableStatistics("person");
    }

    /**
     * Get all used logical databases.
     *
     * @return Array that contains names of all used logical databases.
     *
     */
    public static String[] getUsedLogicalDatabases() {
        String[] str = { getLogicalDBName() };
        return str;
    }


    protected static QueryCacheImpl cache; // cache for PersonDO

    /**
     * Read cache configuration from application configuration file:
     * cache size for org.enhydra.barracuda.discRack.data.person.person table or default cache size.
     */
    public static void readCacheConfiguration() {
        Config tableConfig = null;
        Config cacheConfig = null;
        int maxSize = 0;
        try {
            cacheConfig = (Config)DODS.getDatabaseManager().getConfig().getSection("DB.Cache");
            if (cacheConfig != null) {
                try {
                    tableConfig = cacheConfig.getConfig("person");
                } catch (ConfigException ex) {
                }
                boolean notFound = false;
                if (tableConfig != null) {
                    try {
                        maxSize = tableConfig.getInt("maxCacheSize");
                    } catch (ConfigException ex) {
                        notFound = true;
                    }
                } else
                    notFound = true;
                if ( notFound ) {
                    try {
                        maxSize = cacheConfig.getInt("defaultMaxCacheSize");
                        notFound = false;
                    } catch (ConfigException ex) {
                        notFound = true;
                    }
                }
                if ( notFound ) {
                    maxSize = QueryCache.DEFAULT_MAX_CACHE_SIZE;
                }
            } else
                maxSize = QueryCache.DEFAULT_MAX_CACHE_SIZE;

        } catch ( Exception e ) {
            maxSize = QueryCache.DEFAULT_MAX_CACHE_SIZE;
        }
        if (maxSize != 0) {
            if (cache == null) {
                cache = new QueryCacheImpl(maxSize);
            } else
                cache.setMaxCacheSize(maxSize);
            cache.readConfiguration(tableConfig,cacheConfig);
        } else
            cache = null;

//        System.out.println("Cache size for "+"org.enhydra.barracuda.discRack.data.person.person"+" = "+getMaxCacheSize(true));
    }

    /**
     * Get name of the table that is cached.
     *
     * @return Name of the table that is cached.
     */
    public static String getCacheDodsTableName() {
        return "person";
    }

    /**
     * Get maximum cache size.
     *
     * @return Maximum cache size.
     */
    public static int getMaxCacheSize() {
        if (cache == null)
            return 0;
        return cache.getMaxCacheSize();
    }

    /**
     * Sets maximum cache size.
     *
     * @param maxSize Maximum cache size.
     */
    public static synchronized void setMaxCacheSize(int maxSize) {
        if (maxSize == 0) {
            cache = null;
            refreshStatistics();
            return;
        }
        if (maxSize != 0 && cache == null) {
            cache = new QueryCacheImpl(maxSize);
            try {
                refreshCache();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            cache.setMaxCacheSize(maxSize);
        }
    }

    /**
     * Returns size of currently used cache.
     *
     * @return size of currently used cache.
     */
    public static int getCacheSize() {
        if (cache != null)
            return cache.getCacheSize();
        return 0;
    }

    /**
     * Returns person table cache.
     *
     * @return person table cache.
     */
    public static CacheAdministration getCache() {
        return cache;
    }

    /**
     * Queries all rows in table, and for each row
     * creates a DO instance in the cache.
     * For these DOs, data.readOnly = true,
     * which causes set methods to throw an exception.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public static void refreshCache()
    throws java.sql.SQLException, DatabaseManagerException, ObjectIdException, DataObjectException {
        if (cache != null) {
            cache.refresh();
            String querySnt = cache.getInitialQueryCache();
            if (querySnt != null) {
                PersonQuery query;

                    query = new PersonQuery ();
                    query.hitDatabase();
                    if (!querySnt.equals("*") ) {
                        QueryBuilder builder = query.getQueryBuilder();
                        builder.addWhere(querySnt);
                    }
                    PersonDO obj;
                    try {
                        if (cache.isFull())
                            while ( null != ( obj = query.getNextDO() ) )
                            obj.data.readOnly = true;
                        else
                            query.getNextDO();
                    } catch ( NonUniqueQueryException ex ) {
                        // Since we do not call query.requireUniqueInstance()
                        // this should never happen.
                        throw new ObjectIdException( "Duplicate ObjectId" );
                    }

            }
            refreshStatistics();
        }
    }

    /**
     * Add DO to cache.
     * If DO already exists in cache, just the data member is replaced,
     * so that application references to the DO remain valid.
     * If there  is no caching newDO object is returned.
     *
     * param newDO Data object that will be added to cache.
     *
     * return Data object added to cache.
     */
    public static synchronized PersonDO addToCache( PersonDO newDO ) {
        if (cache == null)
            return newDO;
        PersonDO ret = (PersonDO)cache.addDO(newDO);
        if (ret == null)
            return newDO;
        return ret;
    }

    /**
     * Add DO to cache.
     * If DO already exists in cache, just the data member is replaced,
     * so that application references to the DO remain valid.
     * This method overides method addToCache of the class CoreDO.
     */
    public void addToCache() {
        addToCache(this);
    }

    /**
     */
    public static synchronized PersonDO updateCache( PersonDO newDO ) {
        if (cache == null)
            return newDO;
        PersonDO ret = (PersonDO)cache.updateDO(newDO);
        if (ret == null)
            return newDO;
        return ret;
    }

    /**
     */
    public void updateCache() {
        updateCache(this);
    }

    /**
     */
    public static synchronized PersonDO deleteFromCache( PersonDO DO ) {
        if (cache != null)
            cache.deleteDO(DO);
        return DO;
    }
    /**
     * Remove DO from cache.
     *
     * @param dbName Logical name of the database from which
     * PersonDO object will be removed.
     * @param handle handle of Data object which will be re moved.
     */
    public static synchronized void removeFromCache(String dbName, String handle) {
        if (cache != null) {
            String cacheHandle = ""+handle;

            cache.removeDO( cacheHandle );
        }
    }

    /**
     */
    public void deleteFromCache() {
        deleteFromCache(this);
    }


    /**
     * Remove DO from cache.
     *
     * @param DO Data object which will be removed.
     *
     * @return Removed DO or null if DO object doesn't exist in the cache.
     */
    public static synchronized PersonDO removeFromCache( PersonDO DO ) {
        PersonDO oldDO = null;
        if (cache != null) {
            try {
                oldDO = PersonDO.createVirgin();
                oldDO.setOId(DO.getOId());
                oldDO.setVersion(DO.getVersion());
                oldDO.setData(DO.getData());
                cache.removeDO(DO);
            } catch (Exception e) {
            }
        }
        return oldDO;
    }

    /**
     * Remove DO from cache.
     * This method overides method evict() of the class CoreDO.
     */
    public void evict() {
        if (!isPersistent())
            removeFromCache(this);
    }

    /**
     * Remove DOs from cache.
     *
     * @param DOs Array of DOs which will be removed from cache.
     */
    public static void evict(PersonDO[] DOs) {
        for (int i=0; i<DOs.length; i++)
            DOs[i].evict();
    }

    /**
     * Remove DOs from cache.
     *
     * @param dbName Logical name of the database from which
     * PersonDOs object will be removed.
     * @param handles array of DOs handles that will be removed from cache.
     */
    public static void evict(String dbName, String[] handles) {
        if (handles!=null) {
            for (int i=0; i<handles.length; i++)
                removeFromCache(dbName, handles[i]);
        }
    }

    /**
     * Retrieve contents of the cache as an collection of DO objects.
     *
     * @return Collection of DO objects retrieved from the cache.
     */
    public static Collection getCacheContents() {
        if (cache!=null)
            return cache.getCache().values();
        return null;
    }

    /**
     * Get cache type.
     *
     * @return Cache type.
     */
    public static String getCacheType() {
        if (cache != null)
            return cache.getCacheType();
        return "none";
    }

    /**
     * The cache is used automatically by the class.
     * Callers of the create() methods do not know whether
     * the returned DO instance is shared or not.
     * This prevents the existence of multiple instances in memory
     * of the same database object.
     * The cache is instantiated when the first DO instance is created.
     */
    /**
     * Class that contains unchanging (static) data from the database
     * will have a cache of DOs representing the entire contents of the table.
     */
    static {
        try {
            refreshStatistics();
            readCacheConfiguration();
            refreshCache();
        } catch ( Exception e ) {
            // cannot throw from static block
        }
    }

    /**
     * This method is invoked whenever object needs to be loaded from database.
     *
     * @exception DataObjectException If a data access error occurs.
     */
    public void refresh() throws DataObjectException {
        try {
            loadData();
        } catch ( Exception e ) {
            throw new DataObjectException("Unable to load data for PersonDO id=" + getOId() +
                                          ", error = " + e.getMessage() );
        }
    }

    /**
     * This method is invoked whenever objects needs to be loaded from database.
     *
     * @param DOs Array of DOs which will be red from database.
     *
     * @exception DataObjectException If a data access error occurs.
     */
    public static void refresh(PersonDO[] DOs) throws DataObjectException {
        for (int i=0; i<DOs.length; i++)
            DOs[i].refresh();
    }

    /**
     * createVirgin()
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     *
     * @return Created data object.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static PersonDO createVirgin()
    throws DatabaseManagerException, ObjectIdException {
        return new PersonDO ();
    }


    /**
     * createExisting( BigDecimal )
     *
     * Factory method creates a PersonDO object by searching for it
     * in the database using the passed BigDecimal value as the primary key.
     *
     * Creates a DO that represents an existing entry in the database.
     * Such a DO is used to examine and possibly update such an entry.
     * createExisting() is called only from the code that retrieves
     * an ObjectId from a ResultSet (database query result).
     * createExisting() is protected because no other DO or BO should ever
     * need to call it.
     * FIX unfortunately the createExisting(BigDecimal) form *does* need
     * to be public because it is called by the public ctors of other DOs.
     * For example,
     *    AaaDO contains a ref to a BbbDO,
     *    so there is a method AaaDO.setBbb(BbbDO).
     *    In the ctor AaaDO(ResultSet), we have the call
     *       setBbb( BbbDO.createExisting( rs.getBigDecimal( "bbb", 0 )));
     * Since AaaDO is not in the same package as BbbDO,
     * BbbDO.createExisting(BigDecimal) must be public, not protected.
     * Java needs the C++ 'friend' idea.
     *
     * @param bd The BigDecimal representation of the ObjectId for the object.
     *
     * @return Created PersonDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public static PersonDO createExisting( BigDecimal bd )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == bd )
            return null;
        return createExisting( new ObjectId( bd ) );
    }

    /**
     * createExisting( String )
     *
     * The createExisting method is used to create a <CODE>PersonDO</CODE>
     * from a string handle.
     *
     * @param handle String representation of the ObjectId for the object.
     *
     * @return Created PersonDO object.
     */
    public static PersonDO createExisting( String handle ) {
        PersonDO ret = null;
        try {
            BigDecimal bd = new BigDecimal( handle );
            ret = createExisting( bd );
        } catch ( Exception e ) {
        }
        return ret;
    }

    /**
     * createExisting( ObjectId )
     *
     * Factory method creates a PersonDO object by searching for it
     * in the database using the passed ObjectID value as the primary key.
     *
     * @param id The ObjectId for the object.
     *
     * @return Created PersonDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static PersonDO createExisting( ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == id )
            return null;
        PersonDO ret = null;

        ret = new PersonDO( id );
        ret.setPersistent( true );  // mark DO as persistent (preexisting)
        if ( ! true ) { // If not lazy-loading, fetch DO data now.
            ret.loadData();
        } else {
            statistics.incrementLazyLoadingNum();
        }

        // unset the GenericDO.dirty flag.
        ret.markClean();
        return ret;
    }

    /**
     * createExisting( ResultSet )
     *
     * Factory method used to create an instance of this class to
     * represent a Data Object already existing in the database.
     *
     * @param rs The ResultSet returned by the Query class for
     * an existing Data Object stored in the database.
     *
     * @return Created PersonDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static PersonDO createExisting( ResultSet rs )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == rs )
            return null;
        PersonDO ret = null;
        if ( notUsingOId ) {
            ret = new PersonDO ();
            ret.initFromResultSet( rs );
        } else {
            ret = new PersonDO ( rs );
        }
//        ret = addToCache( ret );
        return ret;
    }

    /**
     * createExisting( RDBRow )
     *
     * Factory method creates a PersonDO object by searching for it
     * in the database using the PersonDO.PrimaryKey value
     * in the passed RDBRow.
     *
     * @param RDBRow A row returned by QueryBuilder.getNextRow().
     *
     * @return Created PersonDO object.
     *
     * @exception DataObjectException
     *   If the RDBRow does not contain a PersonDO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static PersonDO createExisting( RDBRow row )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == row )
            return null;
        RDBColumnValue pk = null;
        try {
            pk = row.get( PersonDO.PrimaryKey );
            return createExisting( pk );
        } catch ( Exception e ) {
            throw new DataObjectException("Cannot create PersonDO, row does not " +
                                          "contain PersonDO primary key." );
        }
    }

    /**
     * createExisting( RDBColumnValue )
     *
     * Factory method creates a PersonDO object by searching for it
     * in the database using the passed PersonDO.PrimaryKey.
     *
     * @param RDBColumnValue a PrimaryKey column value from a row
     * that was returned by QueryBuilder.getNextRow().
     *
     * @return Created PersonDO object.
     *
     * @exception DataObjectException
     *   If the RDBColumnValue does not contain a PersonDO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static PersonDO createExisting( RDBColumnValue pk )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == pk )
            return null;
        if ( ! pk.equals( PersonDO.PrimaryKey ) )
            throw new DataObjectException("Cannot create PersonDO, " +
                                          "RDBColumnValue is not PersonDO.PrimaryKey." );
        BigDecimal bd = null;
        try {
            bd = pk.getBigDecimal();
        } catch ( Exception e ) {
            throw new DataObjectException("Cannot create PersonDO, bad primary key." );
        }
        if ( null == bd )
            return null;
        return createExisting( bd );
    }

    /**
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param data The data struct to copy values from.
     *
     * @return Created PersonDO object.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static PersonDO createCopy( PersonDataStruct data )
    throws DatabaseManagerException, ObjectIdException {
        PersonDO ret = new PersonDO ();
        ret.data = ( PersonDataStruct ) data.duplicate();
        return ret;
    }

    /**
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param orig The original DO to copy.
     *
     * @return Created PersonDO object.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static PersonDO createCopy( PersonDO orig )
    throws DatabaseManagerException, ObjectIdException {
        PersonDO ret = new PersonDO ();
        if ( null != orig.data ) {
            ret.data = ( PersonDataStruct ) orig.data.duplicate();
        }
        return ret;
    }

    /**
     * Causes the DO to refresh itself from the database
     * the next time a set or get method is called.
     */
    public void reload() {
        data = null;
    }

    /**
     * The methods <CODE>
     *     getHandle
     *     hasMatchingHandle

     *     findCachedObjectByHandle

     * </CODE> are used by Presentation Objects that need to populate
     * HTML select lists with Data Objects as options.
     * The <CODE>getHandle()</CODE> method is used
     * to set the value for each option,
     * and the <CODE>hasMatchingHandle()<CODE>

     * methods are used to lookup the Data Object when the selection has
     * been made.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     *
     * @return id of this DO as a string
     *   If an object's id can't be allocated for this object.
     */
    public String  getHandle()
    throws DatabaseManagerException {
        String ret = null;
        if ( null == getOId() )
               throw new DatabaseManagerException( "ID not set " );
        ret = getOId().toString();
        return ret;
    }

    /**
     * Returns cache handle.
     *
     * @return cache handle.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public String  getCacheHandle()
    throws DatabaseManagerException {
        String ret = getHandle();

        return ret;
    }

   /**
     * Compare string version of the id of this DO and handle.
     *
     * @param handle
     *   <CODE>String</CODE> version of DO id.
     *
     * @return boolean
     *   True if the string version of the id of this DO matches passed handle.
     *
     * @see getHandle
     */
    public boolean hasMatchingHandle( String handle ) {
        boolean ret = false;
        if ( null == getOId() )
           return false;
        else {
           String thisHnadle = getOId().toString();
           ret = thisHnadle.equals( handle );
        }
        return ret;
    }

   /**
     * Get data object with key cacheHandle from the cache.
     *
     * @param cacheHandle
     *   <CODE>String</CODE> version of

     * data object's id.
     *
     * @return <CODE>PersonDO</CODE>
     *   Object if one is found in cache, otherwise null.
     *
     * @see getHandle
     */
    public static PersonDO findCachedObjectByHandle( String cacheHandle ){
        if ( null == cacheHandle )
            return null;
        if ( null == cache )
            return null;
        return ( PersonDO ) cache.getDOByHandle( cacheHandle );
    }

    /**
     * Assigns the DataStruct of an existing DO to this DO.
     * Does not duplicate data. Just assigns the reference.
     *
     * @param orig The original DO.
     *
     */
    protected void makeIdentical( PersonDO orig ) {
        super.makeIdentical(orig);
        data = orig.data;
    }

    /**
     * getVersion makes the protected method public in CoreDO.
     *
     * @return Data object's version.
     */
    public int getVersion() {
        return super.getVersion();
    }

    // WebDocWf extension for writeable fully cached objects
    /**
     * Mark the object as read-only.
     *
     * WebDocWf extension
     *
     */
    public void makeReadOnly() {
        if ( null != data )
            data.readOnly=true;
    }

    /**
     * Mark the object as read-write.
     *
     * WebDocWf extension
     */
    public void makeReadWrite() {
        if ( null != data )
            data.readOnly=false;
    }
    // end of WebDocWf extension for writeable fully cached objects


    ////////////////////////// data member Login

    /**
     * static final RDBColumn Login for use with QueryBuilder.
     * See RDBColumn PrimaryKey at the top of this file for usage example.
     */
    static public final RDBColumn Login = new RDBColumn( table, "login", true );

    /**
     * Use for query caching.
     */
    static public final int COLUMN_LOGIN = 5;
        static public final int login_MaxLength = 32;

    /**
     * Get login of the person.
     *
     * @return login of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public String getLogin ()
    throws DataObjectException {
        checkLoad();
        return data.login;
    }
   
    /**
     * Set login of the person.
     *
     * @param login of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void setLogin ( String login )
    throws DataObjectException {
        _setLogin ( login );
    }

    /**
     * _setLogin is a protected method that is called by
     * setLogin if login is not part of
     * a multicolumn foreign key.
     *
     * @param login of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    protected void _setLogin ( String login )
    throws DataObjectException {
        checkLoad();
        data.login =  markNewValue(data.login, login , 0, login_MaxLength, false );
    }


    ////////////////////////// data member Password

    /**
     * static final RDBColumn Password for use with QueryBuilder.
     * See RDBColumn PrimaryKey at the top of this file for usage example.
     */
    static public final RDBColumn Password = new RDBColumn( table, "password", true );

    /**
     * Use for query caching.
     */
    static public final int COLUMN_PASSWORD = 6;
        static public final int password_MaxLength = 32;

    /**
     * Get password of the person.
     *
     * @return password of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public String getPassword ()
    throws DataObjectException {
        checkLoad();
        return data.password;
    }
   
    /**
     * Set password of the person.
     *
     * @param password of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void setPassword ( String password )
    throws DataObjectException {
        _setPassword ( password );
    }

    /**
     * _setPassword is a protected method that is called by
     * setPassword if password is not part of
     * a multicolumn foreign key.
     *
     * @param password of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    protected void _setPassword ( String password )
    throws DataObjectException {
        checkLoad();
        data.password =  markNewValue(data.password, password , 0, password_MaxLength, false );
    }


    ////////////////////////// data member Firstname

    /**
     * static final RDBColumn Firstname for use with QueryBuilder.
     * See RDBColumn PrimaryKey at the top of this file for usage example.
     */
    static public final RDBColumn Firstname = new RDBColumn( table, "firstname", true );

    /**
     * Use for query caching.
     */
    static public final int COLUMN_FIRSTNAME = 7;
        static public final int firstname_MaxLength = 32;

    /**
     * Get firstname of the person.
     *
     * @return firstname of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public String getFirstname ()
    throws DataObjectException {
        checkLoad();
        return data.firstname;
    }
   
    /**
     * Set firstname of the person.
     *
     * @param firstname of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void setFirstname ( String firstname )
    throws DataObjectException {
        _setFirstname ( firstname );
    }

    /**
     * _setFirstname is a protected method that is called by
     * setFirstname if firstname is not part of
     * a multicolumn foreign key.
     *
     * @param firstname of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    protected void _setFirstname ( String firstname )
    throws DataObjectException {
        checkLoad();
        data.firstname =  markNewValue(data.firstname, firstname , 0, firstname_MaxLength, false );
    }


    ////////////////////////// data member Lastname

    /**
     * static final RDBColumn Lastname for use with QueryBuilder.
     * See RDBColumn PrimaryKey at the top of this file for usage example.
     */
    static public final RDBColumn Lastname = new RDBColumn( table, "lastname", true );

    /**
     * Use for query caching.
     */
    static public final int COLUMN_LASTNAME = 8;
        static public final int lastname_MaxLength = 32;

    /**
     * Get lastname of the person.
     *
     * @return lastname of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public String getLastname ()
    throws DataObjectException {
        checkLoad();
        return data.lastname;
    }
   
    /**
     * Set lastname of the person.
     *
     * @param lastname of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void setLastname ( String lastname )
    throws DataObjectException {
        _setLastname ( lastname );
    }

    /**
     * _setLastname is a protected method that is called by
     * setLastname if lastname is not part of
     * a multicolumn foreign key.
     *
     * @param lastname of the person.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    protected void _setLastname ( String lastname )
    throws DataObjectException {
        checkLoad();
        data.lastname =  markNewValue(data.lastname, lastname , 0, lastname_MaxLength, false );
    }

    // array of function objects 
    private static Compare comparators[] = new Compare[9];

    // Initialization function objects
    static {
        comparators[COLUMN_LOGIN] = new Compare() {
            public boolean compare (GenericDO DO, Condition cond) {
                try {
                    return QueryBuilder.compare(((PersonDO)DO).getLogin(),cond.getValue(),cond.getOperator());
                } catch (Exception e) {}
                return false;
            }
        };
        comparators[COLUMN_PASSWORD] = new Compare() {
            public boolean compare (GenericDO DO, Condition cond) {
                try {
                    return QueryBuilder.compare(((PersonDO)DO).getPassword(),cond.getValue(),cond.getOperator());
                } catch (Exception e) {}
                return false;
            }
        };
        comparators[COLUMN_FIRSTNAME] = new Compare() {
            public boolean compare (GenericDO DO, Condition cond) {
                try {
                    return QueryBuilder.compare(((PersonDO)DO).getFirstname(),cond.getValue(),cond.getOperator());
                } catch (Exception e) {}
                return false;
            }
        };
        comparators[COLUMN_LASTNAME] = new Compare() {
            public boolean compare (GenericDO DO, Condition cond) {
                try {
                    return QueryBuilder.compare(((PersonDO)DO).getLastname(),cond.getValue(),cond.getOperator());
                } catch (Exception e) {}
                return false;
            }
        };
    }
    /**
     * Compares whether this DO satisfies condition cond.
     *
     * @param cond condition.
     * 
     * @return true if DO satisfies condition cond, otherwise false.
     */
    public boolean compareCond(Condition cond) {
        try {
            return comparators[cond.getColumnIndex()].compare(this,cond);
        } catch (Exception e) {
        }
        return false;
    }

    /**
     * logicalDbName is logical database name
     * set by setLogicalDBName()
     * and retrieved by getLogicalDBName().
     */
    static private String logicalDbName = null;

    /**
     * setLogicalDBName sets the logical database name that will be used
     * to create DBTransaction and DBQuery objects used by
     * PersonDO and the corresponding Query class.
     * 
     * @param logicalDbNameInConfFile The logical database specified in the
     * application's .conf file.
     *
     * @deprecated It is dangeruous to use this method in multiuser environment because, 
     * this setings are applied to all users (sets logical database to all users)
     */
    static public synchronized void setLogicalDBName( String logicalDbNameInConfFile ) {
        if ( null != logicalDbNameInConfFile && 0    != logicalDbNameInConfFile.length() )
            logicalDbName = logicalDbNameInConfFile;
        else 
            logicalDbName = DODS.getDatabaseManager().getDefaultDB();
    }
    
    /**
     * getLogicalDBName retrieves the logical database name 
     * set by setLogicalDBName().
     *
     * @return the logical database name that was set by method setLogicalDBName()
     *
     * @deprecated 
     */
    static public synchronized String getLogicalDBName() {
        if (logicalDbName == null)
            logicalDbName = DODS.getDatabaseManager().getDefaultDB();                    
        return logicalDbName;
    }

    /**
     * createTransaction() creates a new DBTransaction.
     * This method uses the logical database name set by method setLogicalDBName().
     *
     * If setLogicalDBName() was used to set the logical database name
     * to something other than the value of DatabaseManager.DefaultDatabase
     * in the application's .conf file, then any DBTransaction passed to 
     * save(DBTransaction) or delete(DBTransaction) should be created using 
     * PersonDO.createTransaction().
     *
     * The PersonDO save() and delete() methods use this method.
     *
     * @return A DBTransaction object to use with the PersonDO class.
     */
    static public DBTransaction createTransaction() 
    throws DatabaseManagerException, SQLException {
        DBTransaction ret;
        try {
            ret = DODS.getDatabaseManager().createTransaction(getLogicalDBName());
            ret.setDatabaseName(getLogicalDBName());
            return ret;
        } catch ( DatabaseManagerException e ) { 
            String err = "";
            if ( null != getLogicalDBName() )
                err = "ERROR: Could not create a DBTransaction.  " +
                      "PersonDO.logicalDbName='" + getLogicalDBName() + "'.  "+
                      "The application .conf file must list this name in " +
                      "DatabaseManager.Databases[], and there must be " +
                      "DatabaseManager.DB." + getLogicalDBName() + " settings.";
            throw new DatabaseManagerException( err, e );
        }
    }

    /**
     * createQuery() creates a new DBQuery.
     * This method uses the logical database name set by method setLogicalDBName().
     *
     * If setLogicalDBName() was used to set the logical database name
     * to something other than the value of DatabaseManager.DefaultDatabase
     * in the application's .conf file, then any DBQuery object used to 
     * access the 'person' table should be created using 
     * PersonDO.createQuery().
     *
     * The Query class corresponding to PersonDO uses this method.
     *
     * @return A DBQuery object to use in accessing the 'person' table.
     */
    static public DBQuery createQuery() 
    throws DatabaseManagerException, SQLException {
        try {
            return DODS.getDatabaseManager().createQuery(getLogicalDBName());
        } catch ( DatabaseManagerException e ) { 
            String err = "";
            if ( null != getLogicalDBName() )
                err = "ERROR: Could not create a DBQuery.  " +
                      "PersonDO.logicalDBName='" + getLogicalDBName() + "'.  "+
                      "The application .conf file must list this name in " +
                      "DatabaseManager.Databases[], and there must be " +
                      "DatabaseManager.DB." + getLogicalDBName() + " settings.";
            throw new DatabaseManagerException( err, e );
        }
    }

    /**
     * Protected constructor.
     *
     * @param rs Result set from which to obtain product data.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected PersonDO(ResultSet rs)
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        super(rs);
        initFromResultSet( rs );
        originDatabase = getLogicalDBName();
    }

    /**
     * initFromResultSet initializes the data members of person.
     * This code was separated from the ResultSet constructor
     * so that createExisting(ResultSet) could handle VIEWs.
     *
     * @param rs ResultSet from which data members are initialized.
     */
    private void initFromResultSet( ResultSet rs )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        // Constructing a DO from a ResultSet means we definitely need the 
        // DataStruct ready for the setXxx methods invoked below.
        data = new PersonDataStruct ();
 
        // writeMemberStuff uses the ResultSetExtraction.template
        // to build up the value for this tag:
        // the value is a series of calls to the DO set methods.

            setLogin(  rs.getString( "login"  ) );

            setPassword(  rs.getString( "password"  ) );

            setFirstname(  rs.getString( "firstname"  ) );

            setLastname(  rs.getString( "lastname"  ) );

        markClean();
    }        


    private int[] param = null;

    /**
     * Prepares the statement used to insert this object
     * into the database.
     *
     * @param conn The database connection.
     *
     * @return The insert statement.
     *
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getInsertStatement(DBConnection conn)
    throws SQLException {

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement(  
            "insert into person ( login, password, firstname, lastname, " + getOIdColumnName() + ", " + getVersionColumnName() + " )" +
            "values ( ?, ?, ?, ?, ?, ? )" 
        );

        param = new int[1]; param[0] = 1;
        // writeMemberStuff uses the JDBCsetCalls.template
        // to build up the value for this tag:
        // the value is a series of calls to setPrepStmtParam_TYPE methods.
        // Those methods are defined in GenericDO.
        try {
            setPrepStmtParam_String ( stmt, param, getLogin() ); 
            setPrepStmtParam_String ( stmt, param, getPassword() ); 
            setPrepStmtParam_String ( stmt, param, getFirstname() ); 
            setPrepStmtParam_String ( stmt, param, getLastname() ); 

            /* The ObjecId/Version columns form the primary key.  */
            setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
            setPrepStmtParam_int( stmt, param, getNewVersion() );

        } catch ( Exception e ) {
            throw new SQLException( "Data Object error: " + e.getMessage() );
        }
        statistics.incrementInsertNum();
        return stmt;
    }

    /**
     * Prepares the statement used to update this object
     * in the database.
     *
     * @param conn The database connection
     *
     * @return The update statement.
     *
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getUpdateStatement(DBConnection conn)
    throws SQLException {

        PreparedStatement stmt = conn.prepareStatement(
            "Update person set " + 
            getVersionColumnName() + " = ?, login = ?, password = ?, firstname = ?, lastname = ? " + 
            "where " + getOIdColumnName() + " = ? and " + getVersionColumnName() + " = ?"
        );

        param = new int[1]; param[0] = 1;
        try {
            setPrepStmtParam_int( stmt, param, getNewVersion() );
            setPrepStmtParam_String( stmt, param, getLogin() );
            setPrepStmtParam_String( stmt, param, getPassword() );
            setPrepStmtParam_String( stmt, param, getFirstname() );
            setPrepStmtParam_String( stmt, param, getLastname() );
            setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
            setPrepStmtParam_int(        stmt, param, getVersion() );
        } catch ( Exception e ) {
            throw new SQLException( "Data Object error: " + e.getMessage() );
        }
        statistics.incrementUpdateNum();
        return stmt;
    }

    /**
     * Prepares the statement used to delete this object
     * from the database.
     *
     * @param conn The database connection
     *
     * @return The delete statement.
     *
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getDeleteStatement(DBConnection conn)
    throws SQLException {
        String sql =
            "delete from person \n" +
            "where " + getOIdColumnName() + " = ?";
        PreparedStatement stmt = conn.prepareStatement(sql);
        stmt.setBigDecimal(1, getOId().toBigDecimal());
        statistics.incrementDeleteNum();
        return stmt;
    }


    /*
     * toString - for debugging
     *
     * @return String for debugging.
     *
     */
    public String toString(){
        return toString( 1 );
    }

    public String toString( int indentCount ){
        String indent = "";
        for ( int i = 0; i < indentCount; i++ ) {
            indent += ". ";
        }
        StringBuffer sb = new StringBuffer();
        sb.append(indent + "PersonDO:");

        ObjectId oid = getOId();
        String id = "virgin";
        if ( null != oid )
            id = oid.toString();
        sb.append(" OID=" + id + ",VERSION=" + getVersion());
        if ( null != data ) {    
                sb.append("\n" + indent + "login=" + data.login);
    
                sb.append("\n" + indent + "password=" + data.password);
    
                sb.append("\n" + indent + "firstname=" + data.firstname);
    
                sb.append("\n" + indent + "lastname=" + data.lastname);
;
            sb.append("\n" + indent + "SUPER=" + super.toString( indentCount ));
        }
        return sb.toString(); 
    }

    /**
     * Get array of DiscDO objects that refer to this DO.
     *
     * @return Array of DiscDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public org.enhydra.barracuda.discRack.data.disc.DiscDO[] getDiscDOArray ()
    throws DataObjectException, QueryException {
        org.enhydra.barracuda.discRack.data.disc.DiscDO[] ret = null;
        try {
            org.enhydra.barracuda.discRack.data.disc.DiscQuery q;

                q = new org.enhydra.barracuda.discRack.data.disc.DiscQuery();

            q.setQueryOwner( this, QueryBuilder.EQUAL );
            ret = q.getDOArray();

        } catch ( NonUniqueQueryException e ) {
            throw new DataObjectException("INTERNAL ERROR: unexpected NonUniqueQueryException" );
        } finally {
            if ( null == ret )
            ret = new org.enhydra.barracuda.discRack.data.disc.DiscDO[ 0 ];
        }
        return ret;
    }

    /**
     * Get the single DiscDO object
     * that refers to this DO.
     *
     * @return DiscDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one DiscDO object was found.
     */
    public org.enhydra.barracuda.discRack.data.disc.DiscDO getDiscDO ()
    throws DataObjectException, QueryException, NonUniqueQueryException {
        org.enhydra.barracuda.discRack.data.disc.DiscQuery q;

                q = new org.enhydra.barracuda.discRack.data.disc.DiscQuery();

            q.setQueryOwner( this, QueryBuilder.EQUAL );
            q.requireUniqueInstance();
            return q.getNextDO();

    }

    /**
     * Add (set & commit) a DiscDO object that refers to this DO.
     *
     * @param referrer DiscDO to be set to point to this DO and committed.
     *
     * @exception DatabaseManagerException if could not create a transaction.
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addDiscDO( org.enhydra.barracuda.discRack.data.disc.DiscDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        // WebDocWf extension for DODS row instance security
        // The following line has been changed:
        addDiscDO( referrer, (DBTransaction)null );
        // before: addDiscDO( referrer, null );
        // end of WebDocWf extension for DODS row instance security
    }

    /**
     * Add (set & commit) a DiscDO object that refers to this DO.
     *
     * @param referrer DiscDO to be set to point to this DO and committed.
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction.
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addDiscDO( org.enhydra.barracuda.discRack.data.disc.DiscDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        referrer.setOwner( this );
        referrer.commit( tran );
    }

    /**
     * Remove (delete) a DiscDO object that refers to this DO.
     *
     * @param referrer DiscDO to be deleted.
     *
     * @exception DatabaseManagerException if could not create a transaction.
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeDiscDO( org.enhydra.barracuda.discRack.data.disc.DiscDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        // WebDocWf extension for DODS row instance security
        // The following line has been changed:
        removeDiscDO( referrer, (DBTransaction)null );
        // before: removeDiscDO( referrer, null );
        // end of WebDocWf extension for DODS row instance security
    }

    /**
     * Remove (delete) a DiscDO object that refers to this DO.
     *
     * @param referrer DiscDO to be deleted.
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction.
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeDiscDO( org.enhydra.barracuda.discRack.data.disc.DiscDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        PersonDO referred = referrer.getOwner();
        String referredHandle = referred.getHandle();
        String mydoHandle = this.getHandle();
        if ( null == referredHandle || null == mydoHandle || ( ! referredHandle.equals( mydoHandle ) ) ) {
            throw new DataObjectException( "Object " + referrer +
            " does not refer to object " + this + ", cannot be removed this way." );
        }
        referrer.delete( tran );
    }


    // WebDocWF extension for DODS row counter
    // The following lines have been added:
    /**
     * Get the number of DiscDOs that refer to this DO.
     * via Owner
     *
     * @return The number of objects that refer to this DO.
     *
     * @exception DataObjectException If object is not found in the database.
     *
     * This is a WebDocWf extension for DODS row instance security.
     *
     */
    public int getDiscDOArrayCount ()
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException {
        int ret = 0;
        org.enhydra.barracuda.discRack.data.disc.DiscQuery q;

                q = new org.enhydra.barracuda.discRack.data.disc.DiscQuery();

        q.setQueryOwner( this, QueryBuilder.EQUAL );
        ret = q.getCount();
        return ret;
    }

    /**
     * A stub method for implementing pre-commit assertions
     * for this PersonDO.
     * Implement this stub to throw an RefAssertionException for cases
     * where this object is not valid for writing to the database.
     */
    protected void okToCommit()
    throws RefAssertionException { }

    /**
     * A stub method for implementing pre-delete assertions
     * for this PersonDO.
     * Implement this stub to throw an RefAssertionException for cases
     * where this object is not valid for deletion from the database.
     */
    protected void okToDelete()
    throws RefAssertionException { }

    /**
     * Inserts/Updates the DO into its table.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     *
     * @deprecated Use save() instead.
     */
    public void commit()
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        commit(null);
    }

    /**
     * Inserts/Updates the DO into its table.
     * The transaction is likely provided by the commit() method of another DO
     * which references this DO.
     *
     * @param dbt The transaction object used for this operation.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     *
     * @deprecated Use save() instead.
     */
    public void commit(DBTransaction dbt)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        if (cache != null)
            if (cache.getCacheReadOnly())
                throw new DataObjectException("PersonDO's cache is read-only. Therefore, DML opertions are not allowed.");
        // WebDocWf extension for generic store
        // The following line has been inserted:
        try {
            DBTransaction dbtlocal = dbt;
            boolean needToCommit = false;
            if (dbtlocal == null) {
                dbtlocal = DODS.getDatabaseManager().createTransaction(getOriginDatabase());
                dbtlocal.setDatabaseName(getOriginDatabase());
                needToCommit = true;
            }
            // The following line has been changed:
            modifyDO( dbtlocal, false );
            // before: modifyDO( dbt, false );
            if (needToCommit) {
                dbtlocal.commit();
                dbtlocal.release();
            }
        } catch (DataObjectException e) {
            modifyDO( dbt, false );
        }
        // end of WebDocWf extension for generic store
    }

    /**
     * Inserts/Updates the DO into its table.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     *
     * WebDocWf extension
     */
    public void save()
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        save(null,true);
    }

    /**
     * Inserts/Updates the DO into its table.
     *
     * @param references True if references should be saved with this DO.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     *
     * WebDocWf extension
     */
    public void save(boolean references)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        save(null,references);
    }

    /**
     * Inserts/Updates the DO into its table.
     * The transaction is likely provided by the commit() method of another DO
     * which references this DO.
     *
     * @param dbt The transaction object used for this operation.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     *
     * WebDocWf extension
     */
    public void save(DBTransaction dbt)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        save(dbt, true);
    }

    /**
     * Inserts/Updates the DO into its table.
     * The transaction is likely provided by the commit() method of another DO
     * which references this DO.
     *
     * @param dbt The transaction object used for this operation.
     * @param references True if references of this DO should be saved.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     *
     * WebDocWf extension
     */
    public void save(DBTransaction dbt, boolean references)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        if (cache != null)
            if (cache.getCacheReadOnly())
                throw new DataObjectException("PersonDO's cache is read-only. Therefore, DML opertions are not allowed.");
        // before: modifyDO( dbt, false );
        // The following line has been inserted:
        try {  // Added by Nenad Vico 25.11.2002
            // WebDocWf extension for generic store
            // The following line has been inserted:
            DBTransaction dbtlocal = dbt;
            boolean needToCommit = false;
            if (dbtlocal == null) {
                dbtlocal = DODS.getDatabaseManager().createTransaction(getOriginDatabase());
                dbtlocal.setDatabaseName(getOriginDatabase());
                needToCommit = true;
            }
            // The following line has been changed:
            modifyDO( dbtlocal, false, references );
            if (needToCommit) {
                dbtlocal.commit();
                dbtlocal.release();
            }
        } catch (DataObjectException e) {  // Added by Nenad Vico 25.11.2002
            modifyDO( dbt, false );  // Added by Nenad Vico 25.11.2002
        }  // Added by Nenad Vico 25.11.2002
        // end of WebDocWf extension for generic store
    }

    /**
     * Deletes the DO from its table.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     */
    public void delete()
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        delete((DBTransaction)null);
    }

    /**
     * Deletes the DO from its table.
     * The transaction is likely provided by the delete() method of another DO
     * which references this DO.
     *
     * @param dbt The transaction object used for this operation.
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     */
    public void delete(DBTransaction dbt)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {

        if (cache != null)
            if (cache.getCacheReadOnly())
                throw new DataObjectException("PersonDO's cache is read-only. Therefore, DML opertions are not allowed.");
        // WebDocWf extension for generic store
        // The following lines have been inserted:
        try {       // Added by Nenad Vico 25.11.2002
            DBTransaction dbtlocal = dbt;
            boolean needToCommit = false;
            if (dbtlocal == null) {
                dbtlocal = DODS.getDatabaseManager().createTransaction(getOriginDatabase());
                dbtlocal.setDatabaseName(getOriginDatabase());
                needToCommit = true;
            }
            // The following line has been changed:
            modifyDO( dbtlocal, true );
            // before:  modifyDO( dbt, true );
            // end of WebDocWf extension for generic store
            if (needToCommit) {
                dbtlocal.commit();
                dbtlocal.release();
            }
        } catch (DataObjectException e) { // Added by Nenad Vico 25.11.2002
            modifyDO( dbt, true ); // Added by Nenad Vico 25.11.2002
        } // Added by Nenad Vico 25.11.2002
    }
    
    /**
     * Modifies the DO within its table.
     * Performs recursive commit/delete on referenced DOs;
     * all operations occur within a single transaction
     * to allow rollback in the event of error.
     * Only the creator of the transaction releases it.
     *
     * @param dbt The transaction object used for this operation.
     * @param delete True if doing a delete, otherwise false (for insert/update).
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     * 
     * WebDocWf extension
     */
    protected void modifyDO( DBTransaction dbt, boolean delete)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        modifyDO( dbt, delete, true);
    }

    /**
     * Modifies the DO within its table.
     * Performs recursive commit/delete on referenced DOs;
     * all operations occur within a single transaction
     * to allow rollback in the event of error.
     * Only the creator of the transaction releases it.
     *
     * @param dbt The transaction object used for this operation.
     * @param delete True if doing a delete, otherwise false (for insert/update).
     * @param references True if references should be saved
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     * 
     * WebDocWf extension
     */
    protected void modifyDO( DBTransaction dbt, boolean delete, boolean references )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        if ( delete )
            okToDelete();
        else
            okToCommit();
        boolean ownTransaction = false;
        try {
            if ( null == dbt ) {
                DatabaseManager dbm = DODS.getDatabaseManager();
                dbt = dbm.createTransaction(getOriginDatabase());      // create a transaction
                dbt.setDatabaseName(getOriginDatabase());
                ownTransaction = true;
            } else {

                // WebDocWf fix for circular references
                // The following lines have been inserted
                CoreDO foundDO = dbt.getDO(this);
                if (foundDO!=null) 
                    return;
            }
            // end of WebDocWf fix for circular references
            if ( null == dbt )
                throw new DatabaseManagerException("DatabaseManager.createTransaction returned null." );
            if ( delete ) {
                // Code to perform cascading deletes is generated here
                // if cascading deletes are not supported by the database.      

                // perform cascading delete on referring table
                {
                    org.enhydra.barracuda.discRack.data.disc.DiscDO[] a = getDiscDOArray();
                    for ( int i = 0; i < a.length; i++ ) {
                        a[ i ].delete( dbt );
                    }
                }

                // The following line keeps the compiler happy 
                // when the CASCADING_DELETES tag is empty.
                if ( false )
                    throw new QueryException("XXX");
            } else {
                // WebDocWf extension for save without references
                // The following line has been inserted
                if (references) {
                    // end of WebDocWf extension for save without references
                    // commit referenced DOs.
 
                    // WebDocWf extension for save without references
                    // The following line has been inserted
                }
                // end of WebDocWf extension for save without references
            }
            if ( false ) {
                // This throw is here to keep the compiler happy
                // in the case of a DO that does not refer to other DOs.
                // In that case, the above delete/commit code blocks will be empty
                // and throw nothing.
                throw new DataObjectException( "foo" );   
            }
            if ( delete ) {
              dbt.delete( this );
            } else {
                if ( isLoaded() )
                    dbt.insert( this );   // dbt.insert() handles insertions and updates
            }
            if (ownTransaction) {
                dbt.commit(); // commit the transaction
            }
        } catch (SQLException sqle) {
            StringBuffer message = new StringBuffer("Failed to insert/update DO: ");
            message.append(sqle.getMessage());
            // rollback, if necessary
            if (ownTransaction) {
                try {
                    dbt.rollback();
                } catch (SQLException sqle2) {
                    message.insert(0,"\n");
                    message.insert(0,sqle2.getMessage());
                    message.insert(0,"Rollback failed: ");
                }
            }
            throw new SQLException(message.toString());
        } finally {
            // release the transaction, if any
            if (ownTransaction) {
                dbt.release();
            }
        }
    }
}
